@@ -61,6 +61,11 @@
 #include "existingtransactionmatchfinder.h"
 #include "scheduledtransactionmatchfinder.h"
 
+bool matchNotEmpty(const QString &l, const QString &r)
+{
+  return !l.isEmpty() && QString::compare(l, r, Qt::CaseInsensitive) == 0;
+}
+
 class MyMoneyStatementReader::Private
 {
 public:
@@ -553,11 +558,10 @@ void MyMoneyStatementReader::processSecurityEntry(const MyMoneyStatement::Securi
   QList<MyMoneySecurity> list = file->securityList();
   QList<MyMoneySecurity>::ConstIterator it = list.constBegin();
   while (it != list.constEnd() && security.id().isEmpty()) {
-    if (sec_in.m_strSymbol.isEmpty()) {
-      if ((*it).name() == sec_in.m_strName)
-        security = *it;
-    } else if ((*it).tradingSymbol().toLower() == sec_in.m_strSymbol.toLower())
+    if (matchNotEmpty(sec_in.m_strSymbol, (*it).tradingSymbol()) ||
+        matchNotEmpty(sec_in.m_strName, (*it).name())) {
       security = *it;
+    }
     ++it;
   }
 
@@ -591,7 +595,11 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
   MyMoneyTransaction transactionUnderImport;
 
   QString dbgMsg;
-  dbgMsg = QString("Process %1, '%3', %2").arg(statementTransactionUnderImport.m_datePosted.toString(Qt::ISODate)).arg(statementTransactionUnderImport.m_amount.formatMoney("", 2)).arg(statementTransactionUnderImport.m_strBankID);
+  dbgMsg = QString("Process on: '%1', id: '%3', amount: '%2', fees: '%4'")
+           .arg(statementTransactionUnderImport.m_datePosted.toString(Qt::ISODate))
+           .arg(statementTransactionUnderImport.m_amount.formatMoney("", 2))
+           .arg(statementTransactionUnderImport.m_strBankID)
+           .arg(statementTransactionUnderImport.m_fees.formatMoney("", 2));
   qDebug("%s", qPrintable(dbgMsg));
 
   // mark it imported for the view
@@ -607,7 +615,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
   MyMoneySplit s1;
 
   s1.setMemo(statementTransactionUnderImport.m_strMemo);
-  s1.setValue(statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees);
+  s1.setValue(statementTransactionUnderImport.m_amount + statementTransactionUnderImport.m_fees);
   s1.setShares(s1.value());
   s1.setNumber(statementTransactionUnderImport.m_strNumber);
 
@@ -650,8 +658,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       while (!found && it_account != accounts.constEnd()) {
         QString currencyid = file->account(*it_account).currencyId();
         MyMoneySecurity security = file->security(currencyid);
-        if ((statementTransactionUnderImport.m_strSymbol.toLower() == security.tradingSymbol().toLower())
-            || (statementTransactionUnderImport.m_strSecurity.toLower() == security.name().toLower())) {
+        if (matchNotEmpty(statementTransactionUnderImport.m_strSymbol, security.tradingSymbol()) ||
+            matchNotEmpty(statementTransactionUnderImport.m_strSecurity, security.name())) {
           thisaccount = file->account(*it_account);
           found = true;
 
@@ -695,8 +703,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
           QList<MyMoneySecurity> list = MyMoneyFile::instance()->securityList();
           QList<MyMoneySecurity>::ConstIterator it = list.constBegin();
           while (it != list.constEnd() && security.id().isEmpty()) {
-            if (statementTransactionUnderImport.m_strSymbol.toLower() == (*it).tradingSymbol().toLower()
-                || statementTransactionUnderImport.m_strSecurity.toLower() == (*it).name().toLower()) {
+            if (matchNotEmpty(statementTransactionUnderImport.m_strSymbol, (*it).tradingSymbol()) ||
+                matchNotEmpty(statementTransactionUnderImport.m_strSecurity, (*it).name())) {
               security = *it;
             }
             ++it;
@@ -740,7 +748,8 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
           KMessageBox::information(0, i18n("This imported statement contains investment transactions with no share amount.  These transactions will be ignored."), i18n("No share amount provided"), QString("BlankAmount"));
           return;
         }
-        s1.setPrice(((statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees) / statementTransactionUnderImport.m_shares).convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())));
+        MyMoneyMoney total = -statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees;
+        s1.setPrice((total / statementTransactionUnderImport.m_shares).convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())));
       }
 
 
@@ -777,7 +786,7 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       s2.setAccountId(thisaccount.id());
       transactionUnderImport.addSplit(s2);
 
-      transfervalue = statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees;
+      transfervalue = statementTransactionUnderImport.m_amount + statementTransactionUnderImport.m_fees;
     } else if (statementTransactionUnderImport.m_eAction == MyMoneyStatement::Transaction::eaInterest) {
       if (statementTransactionUnderImport.m_strInterestCategory.isEmpty())
         s1.setAccountId(d->interestId(thisaccount));
@@ -818,10 +827,10 @@ void MyMoneyStatementReader::processTransactionEntry(const MyMoneyStatement::Tra
       if (!statementTransactionUnderImport.m_price.isZero()) {
         s1.setPrice(statementTransactionUnderImport.m_price.abs());
       } else {
-        MyMoneyMoney total;
-        total = -statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees;
-        if (!statementTransactionUnderImport.m_shares.isZero())
+        if (!statementTransactionUnderImport.m_shares.isZero()) {
+          MyMoneyMoney total = -statementTransactionUnderImport.m_amount - statementTransactionUnderImport.m_fees;
           s1.setPrice((total / statementTransactionUnderImport.m_shares).abs().convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())));
+        }
       }
 
       s1.setAction(MyMoneySplit::ActionBuyShares);
