@@ -84,6 +84,9 @@ KEquityPriceUpdateDlg::KEquityPriceUpdateDlg(QWidget *parent, const QString& sec
 
   btnUpdateAll->setEnabled(false);
 
+  btnOK->setGuiItem(KStandardGuiItem::ok());
+  btnCancel->setGuiItem(KStandardGuiItem::cancel());
+
   MyMoneyFile* file = MyMoneyFile::instance();
 
   //
@@ -362,9 +365,8 @@ void KEquityPriceUpdateDlg::storePrices(void)
     }
     ft.commit();
 
-  } catch (MyMoneyException *e) {
+  } catch (const MyMoneyException &) {
     qDebug("Unable to add price information for %s", qPrintable(name));
-    delete e;
   }
 }
 
@@ -393,8 +395,10 @@ void KEquityPriceUpdateDlg::slotUpdateSelectedClicked(void)
     prgOnlineProgress->setMaximum(1 + lvEquityList->invisibleRootItem()->childCount());
     prgOnlineProgress->setValue(skipCnt);
     m_webQuote.launch(item->text(SYMBOL_COL), item->text(ID_COL), item->text(SOURCE_COL));
-  } else
+  } else {
+
     logErrorMessage("No security selected.");
+  }
 }
 
 void KEquityPriceUpdateDlg::slotUpdateAllClicked(void)
@@ -408,8 +412,10 @@ void KEquityPriceUpdateDlg::slotUpdateAllClicked(void)
     prgOnlineProgress->setValue(1);
     m_fUpdateAll = true;
     m_webQuote.launch(item->text(SYMBOL_COL), item->text(ID_COL), item->text(SOURCE_COL));
-  } else
+
+  } else {
     logErrorMessage("Security list is empty.");
+  }
 }
 
 void KEquityPriceUpdateDlg::slotQuoteFailed(const QString& _id, const QString& _symbol)
@@ -443,9 +449,8 @@ void KEquityPriceUpdateDlg::slotQuoteFailed(const QString& _id, const QString& _
       // Re-commit the security
       MyMoneyFile::instance()->modifySecurity(security);
       ft.commit();
-    } catch (MyMoneyException* e) {
-      KMessageBox::error(this, QString("<qt>") + i18n("Cannot update security <b>%1</b>: %2", _symbol, e->what()) + QString("</qt>"), i18n("Price Update Failed"));
-      delete e;
+    } catch (const MyMoneyException &e) {
+      KMessageBox::error(this, QString("<qt>") + i18n("Cannot update security <b>%1</b>: %2", _symbol, e.what()) + QString("</qt>"), i18n("Price Update Failed"));
     }
   }
 
@@ -502,9 +507,8 @@ void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString&
         try {
           sec = MyMoneyFile::instance()->security(id);
           sec = MyMoneyFile::instance()->security(sec.tradingCurrency());
-        } catch (MyMoneyException *e) {
+        } catch (const MyMoneyException &) {
           sec = MyMoneySecurity();
-          delete e;
         }
 
       } else {
@@ -512,9 +516,8 @@ void KEquityPriceUpdateDlg::slotReceivedQuote(const QString& _id, const QString&
         if (splitrx.indexIn(_id) != -1) {
           try {
             sec = MyMoneyFile::instance()->security(splitrx.cap(2).toUtf8());
-          } catch (MyMoneyException *e) {
+          } catch (const MyMoneyException &) {
             sec = MyMoneySecurity();
-            delete e;
           }
         }
       }
@@ -571,6 +574,3 @@ void KEquityPriceUpdateDlg::finishUpdate(void)
 #undef DATE_COL
 #undef ID_COL
 #undef SOURCE_COL
-
-#include "kequitypriceupdatedlg.moc"
-// vim:cin:si:ai:et:ts=2:sw=2:
