@@ -87,7 +87,7 @@ public:
     */
   const QList<MyMoneySplit> getSplits(const MyMoneyTransaction& t) const;
 
-  void setup(const QMap<QString, MyMoneyMoney>& priceInfo);
+  void setup(const QMap<QString, MyMoneyMoney>& priceInfo, int precision);
 
   int currentRow() const;
 
@@ -123,7 +123,15 @@ protected:
     */
   bool isEditMode(void) const;
 
-  void endEdit(bool keyboardDriven);
+  /**
+    * This method returns true if the currently edited split is valid
+    * and can be entered.
+    *
+    * @return true if the split can be entered, false otherwise
+    */
+  bool isEditSplitValid() const;
+
+  void endEdit(bool keyboardDriven, bool setFocusToNextRow = true);
 
 public slots:
   /** No descriptions */
@@ -201,6 +209,11 @@ private:
   MyMoneySplit        m_split;
   MyMoneySplit        m_hiddenSplit;
 
+  /**
+    * This member keeps the precision for the values
+    */
+  int                 m_precision;
+
   /**
     * This member keeps a pointer to the context menu
     */
