@@ -21,8 +21,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config-kmymoney.h>
-
 #include "kreportconfigurationfilterdlg.h"
 
 // ----------------------------------------------------------------------------
@@ -51,7 +49,6 @@
 #include <kguiitem.h>
 #include <kiconloader.h>
 #include <kdebug.h>
-#include <kstdguiitem.h>
 #include <klocale.h>
 
 // ----------------------------------------------------------------------------
@@ -104,8 +101,10 @@ KReportConfigurationFilterDlg::KReportConfigurationFilterDlg(
     m_tab2 = new kMyMoneyReportConfigTab2Decl(m_ui->m_criteriaTab);
     m_tab2->setObjectName("kMyMoneyReportConfigTab2");
     m_ui->m_criteriaTab->insertTab(1, m_tab2, i18n("Rows/Columns"));
-    connect(m_tab2->findChild<KComboBox*>("m_comboRows"), SIGNAL(highlighted(int)), this, SLOT(slotRowTypeChanged(int)));
+    connect(m_tab2->findChild<KComboBox*>("m_comboRows"), SIGNAL(activated(int)), this, SLOT(slotRowTypeChanged(int)));
     connect(m_tab2->findChild<KComboBox*>("m_comboColumns"), SIGNAL(activated(int)), this, SLOT(slotColumnTypeChanged(int)));
+    connect(m_tab2->findChild<KComboBox*>("m_comboRows"), SIGNAL(activated(int)), this, SLOT(slotUpdateColumnsCombo()));
+    connect(m_tab2->findChild<KComboBox*>("m_comboColumns"), SIGNAL(activated(int)), this, SLOT(slotUpdateColumnsCombo()));
     //control the state of the includeTransfer check
     connect(m_ui->m_categoriesView, SIGNAL(stateChanged()), this, SLOT(slotUpdateCheckTransfers()));
 
@@ -195,7 +194,7 @@ void KReportConfigurationFilterDlg::slotSearch(void)
       m_currentState.setMovingAverageDays(m_tab2->findChild<QSpinBox*>("m_movingAverageDays")->value());
     }
   } else if (m_tab3) {
-    MyMoneyReport::ERowType rtq[7] = { MyMoneyReport::eCategory, MyMoneyReport::eTopCategory, MyMoneyReport::ePayee, MyMoneyReport::eAccount, MyMoneyReport::eTopAccount, MyMoneyReport::eMonth, MyMoneyReport::eWeek };
+    MyMoneyReport::ERowType rtq[8] = { MyMoneyReport::eCategory, MyMoneyReport::eTopCategory, MyMoneyReport::eTag, MyMoneyReport::ePayee, MyMoneyReport::eAccount, MyMoneyReport::eTopAccount, MyMoneyReport::eMonth, MyMoneyReport::eWeek };
     m_currentState.setRowType(rtq[m_tab3->findChild<KComboBox*>("m_comboOrganizeBy")->currentIndex()]);
 
     unsigned qc = MyMoneyReport::eQCnone;
@@ -208,6 +207,8 @@ void KReportConfigurationFilterDlg::slotSearch(void)
       qc |= MyMoneyReport::eQCnumber;
     if (m_tab3->findChild<QCheckBox*>("m_checkPayee")->isChecked())
       qc |= MyMoneyReport::eQCpayee;
+    if (m_tab3->findChild<QCheckBox*>("m_checkTag")->isChecked())
+      qc |= MyMoneyReport::eQCtag;
     if (m_tab3->findChild<QCheckBox*>("m_checkCategory")->isChecked())
       qc |= MyMoneyReport::eQCcategory;
     if (m_tab3->findChild<QCheckBox*>("m_checkMemo")->isChecked())
@@ -259,7 +260,17 @@ void KReportConfigurationFilterDlg::slotRowTypeChanged(int row)
 
 void KReportConfigurationFilterDlg::slotColumnTypeChanged(int row)
 {
-  if (m_tab2->findChild<KMyMoneyGeneralCombo*>("m_comboBudget")->isEnabled() && row < 2) {
+  if ((m_tab2->findChild<KMyMoneyGeneralCombo*>("m_comboBudget")->isEnabled() && row < 2)) {
+    m_tab2->findChild<KComboBox*>("m_comboColumns")->setCurrentItem(i18nc("@item the columns will display monthly data", "Monthly"), false);
+  }
+}
+
+void KReportConfigurationFilterDlg::slotUpdateColumnsCombo(void)
+{
+  const int monthlyIndex = 2;
+  const int incomeExpenseIndex = 0;
+  const bool isIncomeExpenseForecast = m_currentState.isIncludingForecast() && m_tab2->findChild<KComboBox*>("m_comboRows")->currentIndex() == incomeExpenseIndex;
+  if (isIncomeExpenseForecast && m_tab2->findChild<KComboBox*>("m_comboColumns")->currentIndex() != monthlyIndex) {
     m_tab2->findChild<KComboBox*>("m_comboColumns")->setCurrentItem(i18nc("@item the columns will display monthly data", "Monthly"), false);
   }
 }
@@ -279,7 +290,12 @@ void KReportConfigurationFilterDlg::slotReset(void)
   m_tab1->findChild<KLineEdit*>("m_editComment")->setText(m_initialState.comment());
   m_tab1->findChild<QCheckBox*>("m_checkCurrency")->setChecked(m_initialState.isConvertCurrency());
   m_tab1->findChild<QCheckBox*>("m_checkFavorite")->setChecked(m_initialState.isFavorite());
-  m_tab1->findChild<QCheckBox*>("m_skipZero")->setChecked(m_initialState.isSkippingZero());
+
+  if (m_initialState.isIncludingPrice() || m_initialState.isSkippingZero()) {
+    m_tab1->findChild<QCheckBox*>("m_skipZero")->setChecked(m_initialState.isSkippingZero());
+  } else {
+    m_tab1->findChild<QCheckBox*>("m_skipZero")->setEnabled(false);
+  }
 
   if (m_tab2) {
     KComboBox *combo = m_tab2->findChild<KComboBox*>("m_comboDetail");
@@ -384,6 +400,9 @@ void KReportConfigurationFilterDlg::slotReset(void)
       case MyMoneyReport::eTopCategory:
         combo->setCurrentItem(i18n("Top Categories"), false);
         break;
+      case MyMoneyReport::eTag:
+        combo->setCurrentItem(i18n("Tags"), false);
+        break;
       case MyMoneyReport::ePayee:
         combo->setCurrentItem(i18n("Payees"), false);
         break;
@@ -400,12 +419,13 @@ void KReportConfigurationFilterDlg::slotReset(void)
         combo->setCurrentItem(i18n("Week"), false);
         break;
       default:
-        throw new MYMONEYEXCEPTION("KReportConfigurationFilterDlg::slotReset(): QueryTable report has invalid rowtype");
+        throw MYMONEYEXCEPTION("KReportConfigurationFilterDlg::slotReset(): QueryTable report has invalid rowtype");
     }
 
     unsigned qc = m_initialState.queryColumns();
     m_tab3->findChild<QCheckBox*>("m_checkNumber")->setChecked(qc & MyMoneyReport::eQCnumber);
     m_tab3->findChild<QCheckBox*>("m_checkPayee")->setChecked(qc & MyMoneyReport::eQCpayee);
+    m_tab3->findChild<QCheckBox*>("m_checkTag")->setChecked(qc & MyMoneyReport::eQCtag);
     m_tab3->findChild<QCheckBox*>("m_checkCategory")->setChecked(qc & MyMoneyReport::eQCcategory);
     m_tab3->findChild<QCheckBox*>("m_checkMemo")->setChecked(qc & MyMoneyReport::eQCmemo);
     m_tab3->findChild<QCheckBox*>("m_checkAccount")->setChecked(qc & MyMoneyReport::eQCaccount);
@@ -437,7 +457,7 @@ void KReportConfigurationFilterDlg::slotReset(void)
         combo->setCurrentItem(m_initialState.chartType());
         break;
       case MyMoneyReport::eChartEnd:
-        throw new MYMONEYEXCEPTION("KReportConfigurationFilterDlg::slotReset(): Report has invalid charttype");
+        throw MYMONEYEXCEPTION("KReportConfigurationFilterDlg::slotReset(): Report has invalid charttype");
     }
     m_tabChart->findChild<QCheckBox*>("m_checkGridLines")->setChecked(m_initialState.isChartGridLines());
     m_tabChart->findChild<QCheckBox*>("m_checkValues")->setChecked(m_initialState.isChartDataLabels());
@@ -557,6 +577,22 @@ void KReportConfigurationFilterDlg::slotReset(void)
     selectAllItems(m_ui->m_payeesView, true);
   }
 
+  //
+  // Tags Filter
+  //
+
+  QStringList tags;
+  if (m_initialState.tags(tags)) {
+    if (tags.empty()) {
+      m_ui->m_emptyTagsButton->setChecked(true);
+    } else {
+      selectAllItems(m_ui->m_tagsView, false);
+      selectItems(m_ui->m_tagsView, tags, true);
+    }
+  } else {
+    selectAllItems(m_ui->m_tagsView, true);
+  }
+
   //
   // Accounts Filter
   //
@@ -629,5 +665,3 @@ void KReportConfigurationFilterDlg::slotUpdateCheckTransfers(void)
     cb->setEnabled(true);
   }
 }
-
-#include "kreportconfigurationfilterdlg.moc"
