@@ -1565,6 +1565,7 @@ void StdTransactionEditor::slotUpdateCategory(const QString& id)
       tabbar->setTabEnabled(KMyMoneyRegister::ActionWithdrawal, true);
     }
 
+    bool disableTransferTab = false;
     if (!id.isEmpty()) {
       MyMoneyAccount acc = MyMoneyFile::instance()->account(id);
       if (acc.isAssetLiability()
@@ -1587,19 +1588,25 @@ void StdTransactionEditor::slotUpdateCategory(const QString& id)
         } else
           categoryLabel->setText(i18n("Transfer to"));
       } else {
-        if (tabbar)
-          tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
+        disableTransferTab = true;
         categoryLabel->setText(i18n("Category"));
       }
       updateAmount(val);
     } else {  //id.isEmpty()
       KMyMoneyCategory* category = dynamic_cast<KMyMoneyCategory*>(m_editWidgets["category"]);
-      if (!category->currentText().isEmpty() && tabbar)
-        tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
+      disableTransferTab = !category->currentText().isEmpty();
       categoryLabel->setText(i18n("Category"));
     }
-    if (tabbar)
+    if (tabbar) {
+      if (disableTransferTab) {
+        // set the proper tab before disabling the currently active tab
+        if (tabbar->currentIndex() == KMyMoneyRegister::ActionTransfer) {
+          tabbar->setCurrentIndex(val.isPositive() ? KMyMoneyRegister::ActionWithdrawal : KMyMoneyRegister::ActionDeposit);
+        }
+        tabbar->setTabEnabled(KMyMoneyRegister::ActionTransfer, false);
+      }
       tabbar->update();
+    }
 
     resizeForm();
   }
@@ -1641,6 +1648,10 @@ void StdTransactionEditor::slotUpdateAmount(const QString& txt)
 
 void StdTransactionEditor::updateAmount(const MyMoneyMoney& val)
 {
+  // we don't do anything if we have multiple transactions selected
+  if (isMultiSelection())
+    return;
+
   QLabel *categoryLabel = dynamic_cast<QLabel*>(haveWidget("category-label"));
   if (categoryLabel) {
     KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(m_editWidgets["cashflow"]);
@@ -1814,11 +1825,6 @@ bool StdTransactionEditor::isComplete(QString& reason) const
     postDate->markAsBadDate();
     postDate->setToolTip("");
   }
-  bool payeeIsPresent = false;
-  bool categoryIsPresent = false;
-  bool amountIsPresent = false;
-  bool memoIsPresent = false;
-  bool dateIsPresent = false;
 
   for (it_w = m_editWidgets.begin(); it_w != m_editWidgets.end(); ++it_w) {
     KMyMoneyPayeeCombo* payee = dynamic_cast<KMyMoneyPayeeCombo*>(*it_w);
@@ -1829,22 +1835,14 @@ bool StdTransactionEditor::isComplete(QString& reason) const
     KMyMoneyCashFlowCombo* cashflow = dynamic_cast<KMyMoneyCashFlowCombo*>(*it_w);
     KTextEdit* memo = dynamic_cast<KTextEdit*>(*it_w);
 
-    if (payee && !(payee->currentText().isEmpty())) {
-      payeeIsPresent = true;
-    }
+    if (payee && !(payee->currentText().isEmpty()))
+      break;
 
-//    if (tagContainer && !(tagContainer->selectedTags().isEmpty()))  //  Tag is not a mandatory field
-//      break;
+    if (category && !category->lineEdit()->text().isEmpty())
+      break;
 
-    //  Mandatory fields
-
-    if (category && !category->lineEdit()->text().isEmpty()) {
-      categoryIsPresent = true;
-    }
-
-    if (amount && !(amount->value().isZero())) {
-      amountIsPresent = true;
-    }
+    if (amount && !(amount->value().isZero()))
+      break;
 
     // the following widgets are only checked if we are editing multiple transactions
     if (isMultiSelection()) {
@@ -1858,24 +1856,17 @@ bool StdTransactionEditor::isComplete(QString& reason) const
       if (cashflow && cashflow->direction() != KMyMoneyRegister::Unknown)
         break;
 
-      if (postDate->date().isValid() && (postDate->date() >= m_account.openingDate())) {
-        dateIsPresent = true;
+      if (postDate->date().isValid() && (postDate->date() >= m_account.openingDate()))
         break;
-      }
 
-      if (memo && m_memoChanged) {
-        memoIsPresent = true;
+      if (memo && m_memoChanged)
         break;
-      }
 
       if (tagContainer && !(tagContainer->selectedTags().isEmpty()))  //  Tag is optional field
         break;
     }
   }
-  bool rc = (categoryIsPresent && amountIsPresent);  //   usual mandatory fields
-  bool rc1 = (payeeIsPresent || categoryIsPresent || amountIsPresent || memoIsPresent) || dateIsPresent;  //  if isMultiSelection(), these are optional
-
-  return (rc || (isMultiSelection() && (it_w != m_editWidgets.end() || rc1)));
+  return it_w != m_editWidgets.end();
 }
 
 void StdTransactionEditor::slotCreateCategory(const QString& name, QString& id)
