@@ -16,10 +16,6 @@
 #ifndef KMYMONEY_H
 #define KMYMONEY_H
 
-#ifdef HAVE_CONFIG_H
-#include <config-kmymoney.h>
-#endif
-
 // ----------------------------------------------------------------------------
 // QT Includes
 
@@ -37,9 +33,10 @@
 
 #include <imymoneyprocessingcalendar.h>
 #include <mymoneyaccount.h>
-#include <mymoneyscheduled.h>
+#include "mymoneyschedule.h"
 #include <mymoneyinstitution.h>
 #include <mymoneypayee.h>
+#include <mymoneytag.h>
 #include <mymoneybudget.h>
 #include <kmymoneyplugin.h>
 #include <register.h>
@@ -278,13 +275,27 @@ protected slots:
 
   /**
     */
-  void slotPayeeNew(const QString& newnameBase, QString& id);
+  bool slotPayeeNew(const QString& newnameBase, QString& id);
   void slotPayeeNew(void);
 
   /**
     */
   void slotPayeeDelete(void);
 
+  /**
+    * Slot that merges two or more selected payess into a new payee
+    */
+  void slotPayeeMerge(void);
+
+  /**
+    */
+  void slotTagNew(const QString& newnameBase, QString& id);
+  void slotTagNew(void);
+
+  /**
+    */
+  void slotTagDelete(void);
+
   /**
     */
   void slotBudgetNew(void);
@@ -517,7 +528,8 @@ public:
   /**
    * This method returns the account for a given @a key - @a value pair.
    * If the account is not found in the list of accounts, MyMoneyAccount()
-   * is returned.
+   * is returned. The @a key - @a value pair can be in the account's kvp
+   * container or the account's online settings kvp container.
    */
   const MyMoneyAccount& account(const QString& key, const QString& value) const;
 
@@ -527,7 +539,7 @@ public:
    */
   void setAccountOnlineParameters(const MyMoneyAccount& acc, const MyMoneyKeyValueContainer& kvps);
 
-  KUrl selectFile(const QString& title, const QString& path, const QString& mask, KFile::Mode mode);
+  KUrl selectFile(const QString& title, const QString& path, const QString& mask, KFile::Mode mode, QWidget *widget);
 
   const MyMoneyAccount& findAccount(const MyMoneyAccount& acc, const MyMoneyAccount& parent) const;
 
@@ -537,7 +549,7 @@ public:
   // otherwise the kidl compiler produces uncompilable results.
   const QString filename() const;
 
-  void webConnect(const QString&, const QByteArray& asn_id);
+  void webConnect(const QString& sourceUrl, const QByteArray& asn_id);
 
   /**
     * Checks if the file with the @a url already exists. If so,
@@ -579,22 +591,15 @@ protected:
    */
   void initStatusBar(void);
 
-  /** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
-   * default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
+  /** queryClose is called by KMainWindow on each closeEvent of a window. Against the
+   * default implementation (only returns true), this calls saveModified() on the document object to ask if the document shall
    * be saved if Modified; on cancel the closeEvent is rejected.
-   * @see KTMainWindow#queryClose
-   * @see KTMainWindow#closeEvent
+   * The settings are saved using saveOptions() if we are about to close.
+   * @see KMainWindow#queryClose
+   * @see QWidget#closeEvent
    */
   virtual bool queryClose(void);
 
-  /** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
-   * Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's
-   * properties.
-   * @see KTMainWindow#queryExit
-   * @see KTMainWindow#closeEvent
-   */
-  virtual bool queryExit(void);
-
   void slotCheckSchedules(void);
 
   virtual void resizeEvent(QResizeEvent*);
@@ -626,6 +631,17 @@ protected:
     */
   bool payeeInList(const QList<MyMoneyPayee>& list, const QString& id) const;
 
+  /**
+    * Check if a list contains a tag with a given id
+    *
+    * @param list const reference to value list
+    * @param id const reference to id
+    *
+    * @retval true object has been found
+    * @retval false object is not in list
+    */
+  bool tagInList(const QList<MyMoneyTag>& list, const QString& id) const;
+
   /**
     * Mark the selected transactions as provided by @a flag. If
     * flag is @a MyMoneySplit::Unknown, the future state depends
@@ -689,9 +705,6 @@ public slots:
   /** */
   void slotFileNew(void);
 
-  /** Open a new window */
-  void slotFileNewWindow(void);
-
   /** open a file and load it into the document*/
   void slotFileOpen(void);
 
@@ -907,6 +920,12 @@ public slots:
     */
   void slotShowPayeeContextMenu(void);
 
+  /**
+    * This slot opens the tag options menu at the current cursor
+    * position.
+    */
+  void slotShowTagContextMenu(void);
+
   /**
     * This slot opens the budget options menu at the current cursor
     * position.
@@ -977,6 +996,8 @@ public slots:
 
   void slotSelectPayees(const QList<MyMoneyPayee>& list);
 
+  void slotSelectTags(const QList<MyMoneyTag>& list);
+
   void slotSelectBudget(const QList<MyMoneyBudget>& list);
 
   void slotSelectTransactions(const KMyMoneyRegister::SelectedTransactions& list);
@@ -1090,6 +1111,11 @@ private:
     */
   int askSaveOnClose(void);
 
+  /**
+    * Implement common task when deleting or merging payees
+    */
+  bool payeeReassign(int type);
+
 signals:
   /**
     * This signal is emitted when a new file is loaded. In the case file
@@ -1105,6 +1131,14 @@ signals:
     */
   void payeesSelected(const QList<MyMoneyPayee>& payees);
 
+  /**
+    * This signal is emitted when a tag/list of tags has been selected by
+    * the GUI. If no tag is selected or the selection is removed,
+    * @p tags is identical to an empty QList. This signal is used
+    * by plugins to get information about changes.
+    */
+  void tagsSelected(const QList<MyMoneyTag>& tags);
+
   /**
     * This signal is emitted when a transaction/list of transactions has been selected by
     * the GUI. If no transaction is selected or the selection is removed,
@@ -1171,6 +1205,9 @@ signals:
   void payeeRename(void);
   void payeeCreated(const QString& id);
 
+  void tagRename(void);
+  void tagCreated(const QString& id);
+
   void currencyRename(void);
   void currencyCreated(const QString& id);
 
