@@ -19,15 +19,11 @@
 #include <QList>
 #include <QtTest/QtTest>
 
-// Include internationalization
-#include <klocale.h>
-#include <KHolidays/Holidays>
-
 #define KMM_MYMONEY_UNIT_TESTABLE friend class MyMoneyScheduleTest;
 
 #include "mymoneysplit.h"
 #include "mymoneymoney.h"
-#include "mymoneyscheduled.h"
+#include "mymoneyschedule.h"
 #include "mymoneyfile.h"
 #include "storage/mymoneyseqaccessmgr.h"
 
@@ -172,8 +168,7 @@ void MyMoneyScheduleTest::testOverdue()
     QVERIFY(sch_overdue.isOverdue() == true);
     QVERIFY(sch_intime.isOverdue() == false);
 
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 }
@@ -212,8 +207,7 @@ void MyMoneyScheduleTest::testNextPayment()
     QVERIFY(sch.nextPayment(QDate(2007, 2, 17)) == QDate(2008, 2, 17));
     QVERIFY(sch.nextPayment(QDate(2007, 2, 18)) == QDate(2008, 2, 17));
 
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 }
@@ -653,8 +647,7 @@ void MyMoneyScheduleTest::testPaymentDates()
     QVERIFY(list[3] == QDate(2027, 3, 24));
     // Would fall on a Saturday so gets moved to 23rd.
     QVERIFY(list[4] == QDate(2029, 3, 23));
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 }
@@ -707,37 +700,88 @@ void MyMoneyScheduleTest::testWriteXML()
   doc.appendChild(el);
   sch.writeXML(doc, el);
 
-  QString ref = QString(
-                  "<!DOCTYPE TEST>\n"
-                  "<SCHEDULE-CONTAINER>\n"
-                  " <SCHEDULED_TX paymentType=\"1\" autoEnter=\"1\" occurenceMultiplier=\"123\" startDate=\"%1\" lastPayment=\"%2\" occurence=\"4\" weekendOption=\"2\" type=\"1\" id=\"SCH0001\" name=\"A Name\" endDate=\"\" fixed=\"1\" >\n" // krazy:exclude=spelling
-                  "  <PAYMENTS>\n"
-                  "   <PAYMENT date=\"%3\" />\n"
-                  "  </PAYMENTS>\n"
-                  "  <TRANSACTION postdate=\"2001-12-28\" commodity=\"EUR\" memo=\"Wohnung:Miete\" id=\"\" entrydate=\"2003-09-29\" >\n"
-                  "   <SPLITS>\n"
-                  "    <SPLIT payee=\"P000001\" reconcileflag=\"2\" shares=\"96379/100\" reconciledate=\"\" action=\"\" bankid=\"\" account=\"A000076\" number=\"\" value=\"96379/100\" memo=\"\" id=\"S0001\" />\n"
-                  "    <SPLIT payee=\"P000001\" reconcileflag=\"1\" shares=\"-96379/100\" reconciledate=\"\" action=\"\" bankid=\"\" account=\"A000276\" number=\"\" value=\"-96379/100\" memo=\"\" id=\"S0002\" />\n"
-                  "   </SPLITS>\n"
-                  "   <KEYVALUEPAIRS>\n"
-                  "    <PAIR key=\"key\" value=\"value\" />\n"
-                  "   </KEYVALUEPAIRS>\n"
-                  "  </TRANSACTION>\n"
-                  " </SCHEDULED_TX>\n"
-                  "</SCHEDULE-CONTAINER>\n"
-                ).arg(QDate::currentDate().toString(Qt::ISODate))
-                .arg(QDate::currentDate().toString(Qt::ISODate))
-                .arg(QDate::currentDate().toString(Qt::ISODate));
+  QCOMPARE(doc.doctype().name(), QLatin1String("TEST"));
+  QDomElement scheduleContainer = doc.documentElement();
+  QVERIFY(scheduleContainer.isElement());
+  QCOMPARE(scheduleContainer.tagName(), QLatin1String("SCHEDULE-CONTAINER"));
+  QVERIFY(scheduleContainer.childNodes().size() == 1);
+  QVERIFY(scheduleContainer.childNodes().at(0).isElement());
 
-#if QT_VERSION >= QT_VERSION_CHECK(4,6,0)
-  ref.replace(QString(" />\n"), QString("/>\n"));
-  ref.replace(QString(" >\n"), QString(">\n"));
-#endif
+  QDomElement schedule = scheduleContainer.childNodes().at(0).toElement();
+  QCOMPARE(schedule.tagName(), QLatin1String("SCHEDULED_TX"));
+  QCOMPARE(schedule.attribute("id"), QLatin1String("SCH0001"));
+  QCOMPARE(schedule.attribute("paymentType"), QLatin1String("1"));
+  QCOMPARE(schedule.attribute("autoEnter"), QLatin1String("1"));
+  QCOMPARE(schedule.attribute("occurenceMultiplier"), QLatin1String("123"));
+  QCOMPARE(schedule.attribute("startDate"), QDate::currentDate().toString(Qt::ISODate));
+  QCOMPARE(schedule.attribute("lastPayment"), QDate::currentDate().toString(Qt::ISODate));
+  QCOMPARE(schedule.attribute("occurenceMultiplier"), QLatin1String("123"));
+  QCOMPARE(schedule.attribute("occurence"), QLatin1String("4"));
+  QCOMPARE(schedule.attribute("type"), QLatin1String("1"));
+  QCOMPARE(schedule.attribute("name"), QLatin1String("A Name"));
+  QCOMPARE(schedule.attribute("fixed"), QLatin1String("1"));
+  QCOMPARE(schedule.attribute("endDate"), QString());
+  QCOMPARE(schedule.childNodes().size(), 2);
 
-  //qDebug("ref = '%s'", qPrintable(ref));
-  //qDebug("doc = '%s'", qPrintable(doc.toString()));
+  QVERIFY(schedule.childNodes().at(0).isElement());
+  QDomElement payments = schedule.childNodes().at(0).toElement();
 
-  QVERIFY(doc.toString() == ref);
+  QVERIFY(schedule.childNodes().at(1).isElement());
+  QDomElement transaction = schedule.childNodes().at(1).toElement();
+  QCOMPARE(transaction.tagName(), QLatin1String("TRANSACTION"));
+  QCOMPARE(transaction.attribute("id"), QString());
+  QCOMPARE(transaction.attribute("postdate"), QLatin1String("2001-12-28"));
+  QCOMPARE(transaction.attribute("commodity"), QLatin1String("EUR"));
+  QCOMPARE(transaction.attribute("memo"), QLatin1String("Wohnung:Miete"));
+  QCOMPARE(transaction.attribute("entrydate"), QLatin1String("2003-09-29"));
+  QCOMPARE(transaction.childNodes().size(), 2);
+
+  QVERIFY(transaction.childNodes().at(0).isElement());
+  QDomElement splits = transaction.childNodes().at(0).toElement();
+  QCOMPARE(splits.tagName(), QLatin1String("SPLITS"));
+  QCOMPARE(splits.childNodes().size(), 2);
+  QVERIFY(splits.childNodes().at(0).isElement());
+  QDomElement split1 = splits.childNodes().at(0).toElement();
+  QCOMPARE(split1.tagName(), QLatin1String("SPLIT"));
+  QCOMPARE(split1.attribute("id"), QLatin1String("S0001"));
+  QCOMPARE(split1.attribute("payee"), QLatin1String("P000001"));
+  QCOMPARE(split1.attribute("reconcileflag"), QLatin1String("2"));
+  QCOMPARE(split1.attribute("shares"), QLatin1String("96379/100"));
+  QCOMPARE(split1.attribute("reconciledate"), QString());
+  QCOMPARE(split1.attribute("action"), QString());
+  QCOMPARE(split1.attribute("bankid"), QString());
+  QCOMPARE(split1.attribute("account"), QLatin1String("A000076"));
+  QCOMPARE(split1.attribute("number"), QString());
+  QCOMPARE(split1.attribute("value"), QLatin1String("96379/100"));
+  QCOMPARE(split1.attribute("memo"), QString());
+  QCOMPARE(split1.childNodes().size(), 0);
+
+  QVERIFY(splits.childNodes().at(1).isElement());
+  QDomElement split2 = splits.childNodes().at(1).toElement();
+  QCOMPARE(split2.tagName(), QLatin1String("SPLIT"));
+  QCOMPARE(split2.attribute("id"), QLatin1String("S0002"));
+  QCOMPARE(split2.attribute("payee"), QLatin1String("P000001"));
+  QCOMPARE(split2.attribute("reconcileflag"), QLatin1String("1"));
+  QCOMPARE(split2.attribute("shares"), QLatin1String("-96379/100"));
+  QCOMPARE(split2.attribute("reconciledate"), QString());
+  QCOMPARE(split2.attribute("action"), QString());
+  QCOMPARE(split2.attribute("bankid"), QString());
+  QCOMPARE(split2.attribute("account"), QLatin1String("A000276"));
+  QCOMPARE(split2.attribute("number"), QString());
+  QCOMPARE(split2.attribute("value"), QLatin1String("-96379/100"));
+  QCOMPARE(split2.attribute("memo"), QString());
+  QCOMPARE(split2.childNodes().size(), 0);
+
+  QDomElement keyValuePairs = transaction.childNodes().at(1).toElement();
+  QCOMPARE(keyValuePairs.tagName(), QLatin1String("KEYVALUEPAIRS"));
+  QCOMPARE(keyValuePairs.childNodes().size(), 1);
+
+  QVERIFY(keyValuePairs.childNodes().at(0).isElement());
+  QDomElement keyValuePair1 = keyValuePairs.childNodes().at(0).toElement();
+  QCOMPARE(keyValuePair1.tagName(), QLatin1String("PAIR"));
+  QCOMPARE(keyValuePair1.attribute("key"), QLatin1String("key"));
+  QCOMPARE(keyValuePair1.attribute("value"), QLatin1String("value"));
+  QCOMPARE(keyValuePair1.childNodes().size(), 0);
 }
 
 void MyMoneyScheduleTest::testReadXML()
@@ -820,8 +864,7 @@ void MyMoneyScheduleTest::testReadXML()
   try {
     sch = MyMoneySchedule(node);
     QFAIL("Missing expected exception");
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
   }
 
   doc.setContent(ref_ok1);
@@ -846,8 +889,7 @@ void MyMoneyScheduleTest::testReadXML()
     QVERIFY(sch.nextDueDate() == sch.lastPayment().addDays(7));
     QVERIFY(sch.recordedPayments().count() == 1);
     QVERIFY(sch.recordedPayments()[0] == QDate::currentDate());
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 
@@ -873,8 +915,7 @@ void MyMoneyScheduleTest::testReadXML()
     QVERIFY(sch.nextDueDate() == sch.lastPayment().addDays(7));
     QVERIFY(sch.recordedPayments().count() == 1);
     QVERIFY(sch.recordedPayments()[0] == QDate::currentDate());
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 }
@@ -912,8 +953,7 @@ void MyMoneyScheduleTest::testHasReferenceTo()
   try {
     sch = MyMoneySchedule(node);
 
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 
@@ -1539,8 +1579,7 @@ void MyMoneyScheduleTest::testPaidEarlyOneTime()
     QVERIFY(sch.isFinished() == true);
     QVERIFY(sch.occurrencePeriod() == MyMoneySchedule::OCCUR_MONTHLY);
     QVERIFY(sch.paymentDates(QDate::currentDate(), QDate::currentDate().addDays(21)).count() == 0);
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 
@@ -1601,8 +1640,7 @@ void MyMoneyScheduleTest::testReplaceId()
     QVERIFY(sch.transaction().splits()[0].accountId() == "A000079");
     QVERIFY(sch.transaction().splits()[1].accountId() == "A000276");
 
-  } catch (MyMoneyException *e) {
-    delete e;
+  } catch (const MyMoneyException &) {
     QFAIL("Unexpected exception");
   }
 
@@ -1612,16 +1650,35 @@ void MyMoneyScheduleTest::testAdjustedWhenItWillEnd()
 {
   MyMoneySchedule s;
 
-  QDate endDate(2011, 8, 13); // this is a nonprocessing day because it's a Saturday
+  QDate endDate(2011, 8, 13); // this is a nonprocessing day because
+  // it's a Saturday
+  QDate refDate(2011, 8, 10); // just some ref date before the last payment
+
   s.setStartDate(endDate.addMonths(-1));
   s.setOccurrence(MyMoneySchedule::OCCUR_MONTHLY);
   s.setEndDate(endDate);
+  // the next due date is on this day but the policy is to move the
+  // schedule to the next processing day (Monday)
   s.setWeekendOption(MyMoneySchedule::MoveAfter);
-  s.setNextDueDate(endDate);  // the next due date is on this day but the policy is to move the schedule to the next processing day
+  s.setNextDueDate(endDate);
 
   // the payment should be found between the respective date and one month after
   QVERIFY(s.paymentDates(endDate, endDate.addMonths(1)).count() == 1);
+
+  // the next payment must be the final one
+  QVERIFY(s.nextPayment(refDate) == endDate);
+
+  // and since it is on a Saturday, the adjusted one must be on the
+  // following Monday
+  QVERIFY(s.adjustedNextPayment(refDate) == endDate.addDays(2));
+
+  // reference for Sunday is still OK
+  QVERIFY(s.adjustedNextPayment(QDate(2011, 8, 14)) == endDate.addDays(2));
+
+  // but it is finished on Monday (as reference date)
+  QVERIFY(!s.adjustedNextPayment(QDate(2011, 8, 15)).isValid());
+
+  // check the # of remaining transactions
+  s.setNextDueDate(endDate.addMonths(-1));
+  QVERIFY(s.transactionsRemaining() == 2);
 }
-
-#include "mymoneyscheduletest.moc"
-
