@@ -20,13 +20,20 @@
 #include <list>
 #include <typeinfo>
 
+// ----------------------------------------------------------------------------
+// QT Includes
+
+// ----------------------------------------------------------------------------
+// KDE Includes
+
+#include <klocalizedstring.h>
+
+// ----------------------------------------------------------------------------
+// Project Includes
+
 #include "../mymoneytransactionfilter.h"
 #include "../mymoneycategory.h"
 
-#define TRY try {
-#define CATCH } catch (MyMoneyException *e) {
-#define PASS } catch (MyMoneyException *e) { throw; }
-
 MyMoneyDatabaseMgr::MyMoneyDatabaseMgr() :
     m_creationDate(QDate::currentDate()),
     m_lastModificationDate(QDate::currentDate()),
@@ -67,7 +74,7 @@ void MyMoneyDatabaseMgr::setUser(const MyMoneyPayee& user)
 {
   m_user = user;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     m_sql->modifyUserInfo(user);
   }
 }
@@ -113,7 +120,7 @@ MyMoneyDatabaseMgr const * MyMoneyDatabaseMgr::duplicate(void)
 void MyMoneyDatabaseMgr::addAccount(MyMoneyAccount& account)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
 
     // create the account.
     MyMoneyAccount newAccount(nextAccountID(), account);
@@ -138,14 +145,14 @@ void MyMoneyDatabaseMgr::addAccount(MyMoneyAccount& parent, MyMoneyAccount& acco
   if (theParent == accountList.constEnd()) {
     QString msg = "Unknown parent account '";
     msg += parent.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   theChild = accountList.constFind(account.id());
   if (theChild == accountList.constEnd()) {
     QString msg = "Unknown child account '";
     msg += account.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   MyMoneyAccount acc = *theParent;
@@ -168,7 +175,7 @@ void MyMoneyDatabaseMgr::addAccount(MyMoneyAccount& parent, MyMoneyAccount& acco
 void MyMoneyDatabaseMgr::addPayee(MyMoneyPayee& payee)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     // create the payee
     MyMoneyPayee newPayee(nextPayeeID(), payee);
 
@@ -183,7 +190,7 @@ const MyMoneyPayee MyMoneyDatabaseMgr::payee(const QString& id) const
   QMap<QString, MyMoneyPayee> payeeList = m_sql->fetchPayees(QStringList(id));
   it = payeeList.constFind(id);
   if (it == payeeList.constEnd())
-    throw new MYMONEYEXCEPTION("Unknown payee '" + id + '\'');
+    throw MYMONEYEXCEPTION("Unknown payee '" + id + '\'');
 
   return *it;
 }
@@ -195,9 +202,11 @@ const MyMoneyPayee MyMoneyDatabaseMgr::payeeByName(const QString& payee) const
 
   QMap<QString, MyMoneyPayee> payeeList;
 
-  TRY
-  payeeList = m_sql->fetchPayees();
-  PASS
+  try {
+    payeeList = m_sql->fetchPayees();
+  } catch (const MyMoneyException &) {
+    throw;
+  }
 
   QMap<QString, MyMoneyPayee>::ConstIterator it_p;
 
@@ -207,7 +216,7 @@ const MyMoneyPayee MyMoneyDatabaseMgr::payeeByName(const QString& payee) const
     }
   }
 
-  throw new MYMONEYEXCEPTION("Unknown payee '" + payee + '\'');
+  throw MYMONEYEXCEPTION("Unknown payee '" + payee + '\'');
 }
 
 void MyMoneyDatabaseMgr::modifyPayee(const MyMoneyPayee& payee)
@@ -218,7 +227,7 @@ void MyMoneyDatabaseMgr::modifyPayee(const MyMoneyPayee& payee)
   it = payeeList.constFind(payee.id());
   if (it == payeeList.constEnd()) {
     QString msg = "Unknown payee '" + payee.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->modifyPayee(payee);
@@ -234,7 +243,7 @@ void MyMoneyDatabaseMgr::removePayee(const MyMoneyPayee& payee)
   it_p = payeeList.constFind(payee.id());
   if (it_p == payeeList.constEnd()) {
     QString msg = "Unknown payee '" + payee.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   // scan all transactions to check if the payee is still referenced
@@ -242,7 +251,7 @@ void MyMoneyDatabaseMgr::removePayee(const MyMoneyPayee& payee)
   QMap<QString, MyMoneyTransaction> transactionList = m_sql->fetchTransactions(); // make sure they're all here
   for (it_t = transactionList.constBegin(); it_t != transactionList.constEnd(); ++it_t) {
     if ((*it_t).hasReferenceTo(payee.id())) {
-      throw new MYMONEYEXCEPTION(QString("Cannot remove payee that is still referenced to a %1").arg("transaction"));
+      throw MYMONEYEXCEPTION(QString("Cannot remove payee that is still referenced to a %1").arg("transaction"));
     }
   }
 
@@ -250,7 +259,7 @@ void MyMoneyDatabaseMgr::removePayee(const MyMoneyPayee& payee)
   QMap<QString, MyMoneySchedule> scheduleList = m_sql->fetchSchedules(); // make sure they're all here
   for (it_s = scheduleList.constBegin(); it_s != scheduleList.constEnd(); ++it_s) {
     if ((*it_s).hasReferenceTo(payee.id())) {
-      throw new MYMONEYEXCEPTION(QString("Cannot remove payee that is still referenced to a %1").arg("schedule"));
+      throw MYMONEYEXCEPTION(QString("Cannot remove payee that is still referenced to a %1").arg("schedule"));
     }
   }
   // remove any reference to report and/or budget
@@ -262,21 +271,125 @@ void MyMoneyDatabaseMgr::removePayee(const MyMoneyPayee& payee)
 const QList<MyMoneyPayee> MyMoneyDatabaseMgr::payeeList(void) const
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     return m_sql->fetchPayees().values();
-  }
-  else
+  } else
     return QList<MyMoneyPayee> ();
 }
 
+void MyMoneyDatabaseMgr::addTag(MyMoneyTag& tag)
+{
+  if (m_sql) {
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
+    // create the tag
+    MyMoneyTag newTag(nextTagID(), tag);
+
+    m_sql->addTag(newTag);
+    tag = newTag;
+  }
+}
+
+const MyMoneyTag MyMoneyDatabaseMgr::tag(const QString& id) const
+{
+  QMap<QString, MyMoneyTag>::ConstIterator it;
+  QMap<QString, MyMoneyTag> tagList = m_sql->fetchTags(QStringList(id));
+  it = tagList.constFind(id);
+  if (it == tagList.constEnd())
+    throw MYMONEYEXCEPTION("Unknown tag '" + id + '\'');
+
+  return *it;
+}
+
+const MyMoneyTag MyMoneyDatabaseMgr::tagByName(const QString& tag) const
+{
+  if (tag.isEmpty())
+    return MyMoneyTag::null;
+
+  QMap<QString, MyMoneyTag> tagList;
+
+  try {
+    tagList = m_sql->fetchTags();
+  } catch (const MyMoneyException &) {
+    throw;
+  }
+
+  QMap<QString, MyMoneyTag>::ConstIterator it_ta;
+
+  for (it_ta = tagList.constBegin(); it_ta != tagList.constEnd(); ++it_ta) {
+    if ((*it_ta).name() == tag) {
+      return *it_ta;
+    }
+  }
+
+  throw MYMONEYEXCEPTION("Unknown tag '" + tag + '\'');
+}
+
+void MyMoneyDatabaseMgr::modifyTag(const MyMoneyTag& tag)
+{
+  QMap<QString, MyMoneyTag> tagList = m_sql->fetchTags(QStringList(tag.id()), true);
+  QMap<QString, MyMoneyTag>::ConstIterator it;
+
+  it = tagList.constFind(tag.id());
+  if (it == tagList.constEnd()) {
+    QString msg = "Unknown tag '" + tag.id() + '\'';
+    throw MYMONEYEXCEPTION(msg);
+  }
+
+  m_sql->modifyTag(tag);
+}
+
+void MyMoneyDatabaseMgr::removeTag(const MyMoneyTag& tag)
+{
+  QMap<QString, MyMoneyTransaction>::ConstIterator it_t;
+  QMap<QString, MyMoneySchedule>::ConstIterator it_s;
+  QMap<QString, MyMoneyTag> tagList = m_sql->fetchTags(QStringList(tag.id()));
+  QMap<QString, MyMoneyTag>::ConstIterator it_ta;
+
+  it_ta = tagList.constFind(tag.id());
+  if (it_ta == tagList.constEnd()) {
+    QString msg = "Unknown tag '" + tag.id() + '\'';
+    throw MYMONEYEXCEPTION(msg);
+  }
+
+  // scan all transactions to check if the tag is still referenced
+
+  QMap<QString, MyMoneyTransaction> transactionList = m_sql->fetchTransactions(); // make sure they're all here
+  for (it_t = transactionList.constBegin(); it_t != transactionList.constEnd(); ++it_t) {
+    if ((*it_t).hasReferenceTo(tag.id())) {
+      throw MYMONEYEXCEPTION(QString("Cannot remove tag that is still referenced to a %1").arg("transaction"));
+    }
+  }
+
+  // check referential integrity in schedules
+  QMap<QString, MyMoneySchedule> scheduleList = m_sql->fetchSchedules(); // make sure they're all here
+  for (it_s = scheduleList.constBegin(); it_s != scheduleList.constEnd(); ++it_s) {
+    if ((*it_s).hasReferenceTo(tag.id())) {
+      throw MYMONEYEXCEPTION(QString("Cannot remove tag that is still referenced to a %1").arg("schedule"));
+    }
+  }
+  // remove any reference to report and/or budget
+  removeReferences(tag.id());
+
+  m_sql->removeTag(tag);
+}
+
+const QList<MyMoneyTag> MyMoneyDatabaseMgr::tagList(void) const
+{
+  if (m_sql) {
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
+    return m_sql->fetchTags().values();
+  } else
+    return QList<MyMoneyTag> ();
+}
+
 const MyMoneyAccount MyMoneyDatabaseMgr::account(const QString& id) const
 {
   if (id.isEmpty()) {
-    throw new MYMONEYEXCEPTION("empty account id");
+    throw MYMONEYEXCEPTION("empty account id");
   }
 
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     QMap <QString, MyMoneyAccount> accountList = m_sql->fetchAccounts(QStringList(id));
     QMap <QString, MyMoneyAccount>::ConstIterator pos = accountList.constFind(id);
 
@@ -284,12 +397,12 @@ const MyMoneyAccount MyMoneyDatabaseMgr::account(const QString& id) const
     if (pos != accountList.constEnd())
       return *pos;
   } else {
-    throw new MYMONEYEXCEPTION("No database connected");
+    throw MYMONEYEXCEPTION("No database connected");
   }
 
   // throw an exception, if it does not exist
   QString msg = "Unknown account id '" + id + '\'';
-  throw new MYMONEYEXCEPTION(msg);
+  throw MYMONEYEXCEPTION(msg);
 }
 
 bool MyMoneyDatabaseMgr::isStandardAccount(const QString& id) const
@@ -304,10 +417,10 @@ bool MyMoneyDatabaseMgr::isStandardAccount(const QString& id) const
 void MyMoneyDatabaseMgr::setAccountName(const QString& id, const QString& name)
 {
   if (!isStandardAccount(id))
-    throw new MYMONEYEXCEPTION("Only standard accounts can be modified using setAccountName()");
+    throw MYMONEYEXCEPTION("Only standard accounts can be modified using setAccountName()");
 
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     startTransaction();
     MyMoneyAccount acc = m_sql->fetchAccounts(QStringList(id), true)[id];
     acc.setName(name);
@@ -319,7 +432,7 @@ void MyMoneyDatabaseMgr::setAccountName(const QString& id, const QString& name)
 void MyMoneyDatabaseMgr::addInstitution(MyMoneyInstitution& institution)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     MyMoneyInstitution newInstitution(nextInstitutionID(), institution);
 
     // mark file as changed
@@ -334,18 +447,29 @@ const QString MyMoneyDatabaseMgr::nextPayeeID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementPayeeId()));
     id = 'P' + id.rightJustified(PAYEE_ID_SIZE, '0');
   }
   return id;
 }
 
+const QString MyMoneyDatabaseMgr::nextTagID(void)
+{
+  QString id;
+  if (m_sql) {
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
+    id.setNum(ulong(m_sql->incrementTagId()));
+    id = 'G' + id.rightJustified(TAG_ID_SIZE, '0');
+  }
+  return id;
+}
+
 const QString MyMoneyDatabaseMgr::nextInstitutionID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementInstitutionId()));
     id = 'I' + id.rightJustified(INSTITUTION_ID_SIZE, '0');
   }
@@ -356,7 +480,7 @@ const QString MyMoneyDatabaseMgr::nextAccountID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementAccountId()));
     id = 'A' + id.rightJustified(ACCOUNT_ID_SIZE, '0');
   }
@@ -367,7 +491,7 @@ const QString MyMoneyDatabaseMgr::nextBudgetID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementBudgetId()));
     id = 'B' + id.rightJustified(BUDGET_ID_SIZE, '0');
   }
@@ -378,7 +502,7 @@ const QString MyMoneyDatabaseMgr::nextReportID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementReportId()));
     id = 'R' + id.rightJustified(REPORT_ID_SIZE, '0');
   }
@@ -389,7 +513,7 @@ const QString MyMoneyDatabaseMgr::nextTransactionID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementTransactionId()));
     id = 'T' + id.rightJustified(TRANSACTION_ID_SIZE, '0');
   }
@@ -400,7 +524,7 @@ const QString MyMoneyDatabaseMgr::nextScheduleID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementScheduleId()));
     id = "SCH" + id.rightJustified(SCHEDULE_ID_SIZE, '0');
   }
@@ -411,7 +535,7 @@ const QString MyMoneyDatabaseMgr::nextSecurityID(void)
 {
   QString id;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     id.setNum(ulong(m_sql->incrementSecurityId()));
     id = 'E' + id.rightJustified(SECURITY_ID_SIZE, '0');
   }
@@ -428,9 +552,9 @@ void MyMoneyDatabaseMgr::addTransaction(MyMoneyTransaction& transaction, const b
 
   // first perform all the checks
   if (!transaction.id().isEmpty())
-    throw new MYMONEYEXCEPTION("transaction already contains an id");
+    throw MYMONEYEXCEPTION("transaction already contains an id");
   if (!transaction.postDate().isValid())
-    throw new MYMONEYEXCEPTION("invalid post date");
+    throw MYMONEYEXCEPTION("invalid post date");
 
   // now check the splits
   foreach (const MyMoneySplit& it_s, transaction.splits()) {
@@ -526,7 +650,7 @@ const MyMoneyInstitution MyMoneyDatabaseMgr::institution(const QString& id) cons
   pos = institutionList.constFind(id);
   if (pos != institutionList.constEnd())
     return *pos;
-  throw new MYMONEYEXCEPTION("unknown institution");
+  throw MYMONEYEXCEPTION("unknown institution");
 }
 
 bool MyMoneyDatabaseMgr::dirty(void) const
@@ -545,7 +669,7 @@ unsigned int MyMoneyDatabaseMgr::accountCount(void) const
 const QList<MyMoneyInstitution> MyMoneyDatabaseMgr::institutionList(void) const
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     return m_sql->fetchInstitutions().values();
   } else {
     return QList<MyMoneyInstitution> ();
@@ -588,12 +712,12 @@ void MyMoneyDatabaseMgr::modifyAccount(const MyMoneyAccount& account, const bool
       commitTransaction();
     } else {
       rollbackTransaction();
-      throw new MYMONEYEXCEPTION("Invalid information for update");
+      throw MYMONEYEXCEPTION("Invalid information for update");
     }
 
   } else {
     rollbackTransaction();
-    throw new MYMONEYEXCEPTION("Unknown account id");
+    throw MYMONEYEXCEPTION("Unknown account id");
   }
 }
 
@@ -607,7 +731,7 @@ void MyMoneyDatabaseMgr::modifyInstitution(const MyMoneyInstitution& institution
   if (pos != institutionList.constEnd()) {
     m_sql->modifyInstitution(institution);
   } else
-    throw new MYMONEYEXCEPTION("unknown institution");
+    throw MYMONEYEXCEPTION("unknown institution");
 }
 
 /**
@@ -633,33 +757,37 @@ void MyMoneyDatabaseMgr::modifyTransaction(const MyMoneyTransaction& transaction
   if (transaction.id().isEmpty()
 //  || transaction.file() != this
       || !transaction.postDate().isValid())
-    throw new MYMONEYEXCEPTION("invalid transaction to be modified");
+    throw MYMONEYEXCEPTION("invalid transaction to be modified");
 
   // now check the splits
   foreach (const MyMoneySplit& it_s, transaction.splits()) {
     // the following lines will throw an exception if the
-    // account or payee do not exist
+    // account, payee or tags do not exist
     account(it_s.accountId());
     if (!it_s.payeeId().isEmpty())
       payee(it_s.payeeId());
+    foreach (const QString& tagId, it_s.tagIdList()) {
+      if (!tagId.isEmpty())
+        tag(tagId);
+    }
   }
 
   // new data seems to be ok. find old version of transaction
   // in our pool. Throw exception if unknown.
 //  if(!m_transactionKeys.contains(transaction.id()))
-//    throw new MYMONEYEXCEPTION("invalid transaction id");
+//    throw MYMONEYEXCEPTION("invalid transaction id");
 
 //  QString oldKey = m_transactionKeys[transaction.id()];
   QMap <QString, MyMoneyTransaction> transactionList = m_sql->fetchTransactions("('" + transaction.id() + "')");
 //  if(transactionList.size() != 1)
-//    throw new MYMONEYEXCEPTION("invalid transaction key");
+//    throw MYMONEYEXCEPTION("invalid transaction key");
 
   QMap<QString, MyMoneyTransaction>::ConstIterator it_t;
 
 //  it_t = transactionList.find(oldKey);
   it_t = transactionList.constBegin();
   if (it_t == transactionList.constEnd())
-    throw new MYMONEYEXCEPTION("invalid transaction key");
+    throw MYMONEYEXCEPTION("invalid transaction key");
 
   m_sql->modifyTransaction(transaction);
 
@@ -701,7 +829,7 @@ void MyMoneyDatabaseMgr::modifyTransaction(const MyMoneyTransaction& transaction
 void MyMoneyDatabaseMgr::reparentAccount(MyMoneyAccount &account, MyMoneyAccount& parent)
 {
   if (account.accountType() == MyMoneyAccount::Stock && parent.accountType() != MyMoneyAccount::Investment)
-    throw new MYMONEYEXCEPTION("Cannot move a stock acocunt into a non-investment account");
+    throw MYMONEYEXCEPTION("Cannot move a stock acocunt into a non-investment account");
 
   QStringList accountIdList;
   QMap<QString, MyMoneyAccount>::ConstIterator oldParent;
@@ -754,20 +882,20 @@ void MyMoneyDatabaseMgr::removeTransaction(const MyMoneyTransaction& transaction
 
   // first perform all the checks
   if (transaction.id().isEmpty())
-    throw new MYMONEYEXCEPTION("invalid transaction to be deleted");
+    throw MYMONEYEXCEPTION("invalid transaction to be deleted");
 
   QMap<QString, QString>::ConstIterator it_k;
   QMap<QString, MyMoneyTransaction>::ConstIterator it_t;
 
 //  it_k = m_transactionKeys.find(transaction.id());
 //  if(it_k == m_transactionKeys.end())
-//    throw new MYMONEYEXCEPTION("invalid transaction to be deleted");
+//    throw MYMONEYEXCEPTION("invalid transaction to be deleted");
 
   QMap <QString, MyMoneyTransaction> transactionList = m_sql->fetchTransactions("('" + QString(transaction.id()) + "')");
 //  it_t = transactionList.find(*it_k);
   it_t = transactionList.constBegin();
   if (it_t == transactionList.constEnd())
-    throw new MYMONEYEXCEPTION("invalid transaction key");
+    throw MYMONEYEXCEPTION("invalid transaction key");
 
   // scan the splits and collect all accounts that need
   // to be updated after the removal of this transaction
@@ -822,24 +950,29 @@ void MyMoneyDatabaseMgr::transactionList(QList<MyMoneyTransaction>& list, MyMone
 {
   list.clear();
 
-  TRY
-  if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
-    list = m_sql->fetchTransactions(filter).values();
+  try {
+    if (m_sql) {
+      if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
+      list = m_sql->fetchTransactions(filter).values();
+    }
+  } catch (const MyMoneyException &) {
+    throw;
   }
-  PASS
 }
 
 void MyMoneyDatabaseMgr::transactionList(QList<QPair<MyMoneyTransaction, MyMoneySplit> >& list, MyMoneyTransactionFilter& filter) const
 {
   list.clear();
   MyMoneyMap<QString, MyMoneyTransaction> transactionList;
-  TRY
-  if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
-    transactionList = m_sql->fetchTransactions(filter);
+  try {
+    if (m_sql) {
+      if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
+      transactionList = m_sql->fetchTransactions(filter);
+    }
+  } catch (const MyMoneyException &) {
+    throw;
   }
-  PASS
+
 
   QMap<QString, MyMoneyTransaction>::ConstIterator it_t;
   QMap<QString, MyMoneyTransaction>::ConstIterator txEnd = transactionList.end();
@@ -864,10 +997,10 @@ void MyMoneyDatabaseMgr::removeAccount(const MyMoneyAccount& account)
 
   // check that it's not one of the standard account groups
   if (isStandardAccount(account.id()))
-    throw new MYMONEYEXCEPTION("Unable to remove the standard account groups");
+    throw MYMONEYEXCEPTION("Unable to remove the standard account groups");
 
   if (hasActiveSplits(account.id())) {
-    throw new MYMONEYEXCEPTION("Unable to remove account with active splits");
+    throw MYMONEYEXCEPTION("Unable to remove account with active splits");
   }
 
   // re-parent all sub-ordinate accounts to the parent of the account
@@ -892,14 +1025,14 @@ void MyMoneyDatabaseMgr::removeAccount(const MyMoneyAccount& account)
 
   it_a = accountList.constFind(account.id());
   if (it_a == accountList.constEnd())
-    throw new MYMONEYEXCEPTION("Internal error: account not found in list");
+    throw MYMONEYEXCEPTION("Internal error: account not found in list");
 
   it_p = accountList.constFind(parent.id());
   if (it_p == accountList.constEnd())
-    throw new MYMONEYEXCEPTION("Internal error: parent account not found in list");
+    throw MYMONEYEXCEPTION("Internal error: parent account not found in list");
 
   if (!account.institutionId().isEmpty())
-    throw new MYMONEYEXCEPTION("Cannot remove account still attached to an institution");
+    throw MYMONEYEXCEPTION("Cannot remove account still attached to an institution");
 
   // FIXME: check referential integrity for the account to be removed
 
@@ -944,7 +1077,7 @@ void MyMoneyDatabaseMgr::removeInstitution(const MyMoneyInstitution& institution
     // mark file as changed
     m_sql->removeInstitution(institution);
   } else
-    throw new MYMONEYEXCEPTION("invalid institution");
+    throw MYMONEYEXCEPTION("invalid institution");
 }
 
 const MyMoneyTransaction MyMoneyDatabaseMgr::transaction(const QString& id) const
@@ -952,7 +1085,7 @@ const MyMoneyTransaction MyMoneyDatabaseMgr::transaction(const QString& id) cons
   // get the full key of this transaction, throw exception
   // if it's invalid (unknown)
   //if(!m_transactionKeys.contains(id))
-  //  throw new MYMONEYEXCEPTION("invalid transaction id");
+  //  throw MYMONEYEXCEPTION("invalid transaction id");
 
   // check if this key is in the list, throw exception if not
   //QString key = m_transactionKeys[id];
@@ -962,7 +1095,7 @@ const MyMoneyTransaction MyMoneyDatabaseMgr::transaction(const QString& id) cons
   //return the first element.
   //if(!transactionList.contains(key))
   if (!transactionList.size())
-    throw new MYMONEYEXCEPTION("invalid transaction key");
+    throw MYMONEYEXCEPTION("invalid transaction key");
 
   return transactionList.begin().value();
 }
@@ -1055,7 +1188,7 @@ const MyMoneyTransaction MyMoneyDatabaseMgr::transaction(const QString& account,
     // find account object in list, throw exception if unknown
     acc = m_accountList.find(account);
     if(acc == m_accountList.end())
-      throw new MYMONEYEXCEPTION("unknown account id");
+      throw MYMONEYEXCEPTION("unknown account id");
 
     // get the transaction info from the account
     MyMoneyAccount::Transaction t = (*acc).transaction(idx);
@@ -1078,7 +1211,7 @@ const MyMoneyTransaction MyMoneyDatabaseMgr::transaction(const QString& account,
 
   transactionList(list, filter);
   if (idx < 0 || idx >= static_cast<int>(list.count()))
-    throw new MYMONEYEXCEPTION("Unknown idx for transaction");
+    throw MYMONEYEXCEPTION("Unknown idx for transaction");
 
   return transaction(list[idx].id());
 }
@@ -1092,7 +1225,7 @@ void MyMoneyDatabaseMgr::accountList(QList<MyMoneyAccount>& list) const
 {
   QMap <QString, MyMoneyAccount> accountList;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     accountList  = m_sql->fetchAccounts();
   }
   QMap<QString, MyMoneyAccount>::ConstIterator it;
@@ -1154,7 +1287,7 @@ void MyMoneyDatabaseMgr::modifySecurity(const MyMoneySecurity& security)
   if (it == securitiesList.constEnd()) {
     QString msg = "Unknown security  '";
     msg += security.id() + "' during modifySecurity()";
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->modifySecurity(security);
@@ -1171,7 +1304,7 @@ void MyMoneyDatabaseMgr::removeSecurity(const MyMoneySecurity& security)
   if (it == securitiesList.constEnd()) {
     QString msg = "Unknown security  '";
     msg += security.id() + "' during removeSecurity()";
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->removeSecurity(security);
@@ -1227,13 +1360,13 @@ void MyMoneyDatabaseMgr::addSchedule(MyMoneySchedule& sched)
 {
   // first perform all the checks
   if (!sched.id().isEmpty())
-    throw new MYMONEYEXCEPTION("schedule already contains an id");
+    throw MYMONEYEXCEPTION("schedule already contains an id");
 
   // The following will throw an exception when it fails
   sched.validate(false);
 
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     try {
       startTransaction();
       sched = MyMoneySchedule(nextScheduleID(), sched);
@@ -1255,7 +1388,7 @@ void MyMoneyDatabaseMgr::modifySchedule(const MyMoneySchedule& sched)
   it = scheduleList.constFind(sched.id());
   if (it == scheduleList.constEnd()) {
     QString msg = "Unknown schedule '" + sched.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->modifySchedule(sched);
@@ -1269,7 +1402,7 @@ void MyMoneyDatabaseMgr::removeSchedule(const MyMoneySchedule& sched)
   it = scheduleList.constFind(sched.id());
   if (it == scheduleList.constEnd()) {
     QString msg = "Unknown schedule '" + sched.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   // FIXME: check referential integrity for loan accounts
@@ -1289,7 +1422,7 @@ const MyMoneySchedule MyMoneyDatabaseMgr::schedule(const QString& id) const
 
   // throw an exception, if it does not exist
   QString msg = "Unknown schedule id '" + id + '\'';
-  throw new MYMONEYEXCEPTION(msg);
+  throw MYMONEYEXCEPTION(msg);
 }
 
 const QList<MyMoneySchedule> MyMoneyDatabaseMgr::scheduleList(const QString& accountId,
@@ -1302,7 +1435,7 @@ const QList<MyMoneySchedule> MyMoneyDatabaseMgr::scheduleList(const QString& acc
 {
   QMap<QString, MyMoneySchedule> scheduleList;
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     scheduleList = m_sql->fetchSchedules();
   }
   QMap<QString, MyMoneySchedule>::ConstIterator pos;
@@ -1429,13 +1562,13 @@ const QList<MyMoneySchedule> MyMoneyDatabaseMgr::scheduleListEx(int scheduleType
 void MyMoneyDatabaseMgr::addCurrency(const MyMoneySecurity& currency)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     QMap<QString, MyMoneySecurity> currencyList = m_sql->fetchCurrencies(QStringList(currency.id()));
     QMap<QString, MyMoneySecurity>::ConstIterator it;
 
     it = currencyList.constFind(currency.id());
     if (it != currencyList.constEnd()) {
-      throw new MYMONEYEXCEPTION(QString("Cannot add currency with existing id %1").arg(currency.id()));
+      throw MYMONEYEXCEPTION(i18n("Cannot add currency with existing id %1", currency.id()));
     }
 
     m_sql->addCurrency(currency);
@@ -1449,7 +1582,7 @@ void MyMoneyDatabaseMgr::modifyCurrency(const MyMoneySecurity& currency)
 
   it = currencyList.constFind(currency.id());
   if (it == currencyList.constEnd()) {
-    throw new MYMONEYEXCEPTION(QString("Cannot modify currency with unknown id %1").arg(currency.id()));
+    throw MYMONEYEXCEPTION(i18n("Cannot modify currency with unknown id %1", currency.id()));
   }
 
   m_sql->modifyCurrency(currency);
@@ -1464,7 +1597,7 @@ void MyMoneyDatabaseMgr::removeCurrency(const MyMoneySecurity& currency)
 
   it = currencyList.constFind(currency.id());
   if (it == currencyList.constEnd()) {
-    throw new MYMONEYEXCEPTION(QString("Cannot remove currency with unknown id %1").arg(currency.id()));
+    throw MYMONEYEXCEPTION(i18n("Cannot remove currency with unknown id %1", currency.id()));
   }
 
   m_sql->removeCurrency(currency);
@@ -1480,7 +1613,7 @@ const MyMoneySecurity MyMoneyDatabaseMgr::currency(const QString& id) const
 
   it = currencyList.constFind(id);
   if (it == currencyList.constEnd()) {
-    throw new MYMONEYEXCEPTION(QString("Cannot retrieve currency with unknown id '%1'").arg(id));
+    throw MYMONEYEXCEPTION(i18n("Cannot retrieve currency with unknown id '%1'", id));
   }
 
   return *it;
@@ -1489,7 +1622,7 @@ const MyMoneySecurity MyMoneyDatabaseMgr::currency(const QString& id) const
 const QList<MyMoneySecurity> MyMoneyDatabaseMgr::currencyList(void) const
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     return m_sql->fetchCurrencies().values();
   } else {
     return QList<MyMoneySecurity> ();
@@ -1499,7 +1632,7 @@ const QList<MyMoneySecurity> MyMoneyDatabaseMgr::currencyList(void) const
 const QList<MyMoneyReport> MyMoneyDatabaseMgr::reportList(void) const
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     return m_sql->fetchReports().values();
   } else {
     return QList<MyMoneyReport> ();
@@ -1509,7 +1642,7 @@ const QList<MyMoneyReport> MyMoneyDatabaseMgr::reportList(void) const
 void MyMoneyDatabaseMgr::addReport(MyMoneyReport& report)
 {
   if (!report.id().isEmpty())
-    throw new MYMONEYEXCEPTION("transaction already contains an id");
+    throw MYMONEYEXCEPTION("transaction already contains an id");
 
   m_sql->addReport(MyMoneyReport(nextReportID(), report));
 }
@@ -1522,7 +1655,7 @@ void MyMoneyDatabaseMgr::modifyReport(const MyMoneyReport& report)
   it = reportList.constFind(report.id());
   if (it == reportList.constEnd()) {
     QString msg = "Unknown report '" + report.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->modifyReport(report);
@@ -1546,7 +1679,7 @@ void MyMoneyDatabaseMgr::removeReport(const MyMoneyReport& report)
   it = reportList.constFind(report.id());
   if (it == reportList.constEnd()) {
     QString msg = "Unknown report '" + report.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
 
   m_sql->removeReport(report);
@@ -1574,7 +1707,7 @@ const MyMoneyBudget MyMoneyDatabaseMgr::budgetByName(const QString& budget) cons
     }
   }
 
-  throw new MYMONEYEXCEPTION("Unknown budget '" + budget + '\'');
+  throw MYMONEYEXCEPTION("Unknown budget '" + budget + '\'');
 }
 
 void MyMoneyDatabaseMgr::modifyBudget(const MyMoneyBudget& budget)
@@ -1584,14 +1717,14 @@ void MyMoneyDatabaseMgr::modifyBudget(const MyMoneyBudget& budget)
   //it = m_budgetList.find(budget.id());
   //if(it == m_budgetList.end()) {
   //  QString msg = "Unknown budget '" + budget.id() + '\'';
-  //  throw new MYMONEYEXCEPTION(msg);
+  //  throw MYMONEYEXCEPTION(msg);
   //}
   //m_budgetList.modify(budget.id(), budget);
 
   startTransaction();
   if (m_sql->fetchBudgets(QStringList(budget.id()), true).empty()) {
     QString msg = "Unknown budget '" + budget.id() + '\'';
-    throw new MYMONEYEXCEPTION(msg);
+    throw MYMONEYEXCEPTION(msg);
   }
   m_sql->modifyBudget(budget);
   commitTransaction();
@@ -1614,7 +1747,7 @@ void MyMoneyDatabaseMgr::removeBudget(const MyMoneyBudget& budget)
 //  it = m_budgetList.find(budget.id());
 //  if(it == m_budgetList.end()) {
 //    QString msg = "Unknown budget '" + budget.id() + '\'';
-//    throw new MYMONEYEXCEPTION(msg);
+//    throw MYMONEYEXCEPTION(msg);
 //  }
 //
   m_sql->removeBudget(budget);
@@ -1691,6 +1824,10 @@ bool MyMoneyDatabaseMgr::isReferenced(const MyMoneyObject& obj, const MyMoneyFil
     QList<MyMoneyPayee> payeeList = MyMoneyFile::instance()->payeeList();
     rc = (payeeList.end() != std::find_if(payeeList.begin(), payeeList.end(),  isReferencedHelper(id)));
   }
+  if (!skipCheck[RefCheckTag] && !rc) {
+    QList<MyMoneyTag> tagList = MyMoneyFile::instance()->tagList();
+    rc = (tagList.end() != std::find_if(tagList.begin(), tagList.end(),  isReferencedHelper(id)));
+  }
   if (!skipCheck[RefCheckReport] && !rc) {
     QMap<QString, MyMoneyReport> reportList = m_sql->fetchReports();
     rc = (reportList.end() != std::find_if(reportList.begin(), reportList.end(),  isReferencedHelper(id)));
@@ -1735,7 +1872,7 @@ void MyMoneyDatabaseMgr::close(void)
 void MyMoneyDatabaseMgr::startTransaction(void)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     m_sql->startCommitUnit("databasetransaction");
   }
 }
@@ -1743,7 +1880,7 @@ void MyMoneyDatabaseMgr::startTransaction(void)
 bool MyMoneyDatabaseMgr::commitTransaction(void)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     return m_sql->endCommitUnit("databasetransaction");
   }
   return false;
@@ -1752,7 +1889,7 @@ bool MyMoneyDatabaseMgr::commitTransaction(void)
 void MyMoneyDatabaseMgr::rollbackTransaction(void)
 {
   if (m_sql) {
-    if (! m_sql->isOpen()) ((QSqlDatabase*)(m_sql.data()))->open();
+    if (! m_sql->isOpen())((QSqlDatabase*)(m_sql.data()))->open();
     m_sql->cancelCommitUnit("databasetransaction");
   }
 }
@@ -1830,6 +1967,11 @@ void MyMoneyDatabaseMgr::loadPayees(const QMap<QString, MyMoneyPayee>& /*map*/)
 //  m_payeeList = map;
 }
 
+void MyMoneyDatabaseMgr::loadTags(const QMap<QString, MyMoneyTag>& /*map*/)
+{
+//  m_tagList = map;
+}
+
 void MyMoneyDatabaseMgr::loadSchedules(const QMap<QString, MyMoneySchedule>& /*map*/)
 {
 //  m_scheduleList = map;
@@ -1896,6 +2038,11 @@ unsigned long MyMoneyDatabaseMgr::payeeId(void) const
   return m_sql->getNextPayeeId();
 }
 
+unsigned long MyMoneyDatabaseMgr::tagId(void) const
+{
+  return m_sql->getNextTagId();
+}
+
 unsigned long MyMoneyDatabaseMgr::institutionId(void) const
 {
   return m_sql->getNextInstitutionId();
@@ -1936,6 +2083,11 @@ void MyMoneyDatabaseMgr::loadPayeeId(const unsigned long id)
   m_sql->loadPayeeId(id);
 }
 
+void MyMoneyDatabaseMgr::loadTagId(const unsigned long id)
+{
+  m_sql->loadTagId(id);
+}
+
 void MyMoneyDatabaseMgr::loadInstitutionId(const unsigned long id)
 {
   m_sql->loadInstitutionId(id);
@@ -2004,7 +2156,3 @@ void MyMoneyDatabaseMgr::removeReferences(const QString& id)
 //    budgetList.modify(b.id(), b);
   }
 }
-
-#undef TRY
-#undef CATCH
-#undef PASS
