@@ -124,6 +124,13 @@ public:
   virtual const QString intString(const MyMoneyDbIntColumn& c) const;
   virtual bool requiresExternalFile() const;
   virtual bool requiresCreation() const;
+  virtual bool isPasswordSupported() const;
+};
+
+class MyMoneySqlCipher3Driver : public MyMoneySqlite3Driver
+{
+public:
+  virtual bool isPasswordSupported() const;
 };
 
 //********************* The driver map *********************
@@ -140,7 +147,7 @@ const QMap<QString, QString> MyMoneyDbDriver::driverMap()
   map["QPSQL"] = QString("PostgreSQL v7.3 and up");
   map["QTDS"] = QString("Sybase Adaptive Server and Microsoft SQL Server");
   map["QSQLITE"] = QString("SQLite Version 3");
-
+  map["SQLCIPHER"] = QString("SQLCipher Version 3 (encrypted SQLite)");
   return map;
 }
 
@@ -163,7 +170,9 @@ KSharedPtr<MyMoneyDbDriver> MyMoneyDbDriver::create(const QString& type)
     return KSharedPtr<MyMoneyDbDriver> (new MyMoneySybaseDriver());
   else if (type == "QSQLITE")
     return KSharedPtr<MyMoneyDbDriver> (new MyMoneySqlite3Driver());
-  else throw new MYMONEYEXCEPTION(QString("Unknown database driver type").arg(type));
+  else if (type == "SQLCIPHER")
+    return KSharedPtr<MyMoneyDbDriver> (new MyMoneySqlCipher3Driver());
+  else throw MYMONEYEXCEPTION(QString("Unknown database driver type").arg(type));
 }
 
 MyMoneyDbDriver::MyMoneyDbDriver()
@@ -621,6 +630,23 @@ const QString MyMoneyDbDriver::tableOptionString() const
   return "";
 }
 
+//*************************************************
+// Define
+bool MyMoneyDbDriver::isPasswordSupported() const
+{
+  return true;
+}
+
+bool MyMoneySqlite3Driver::isPasswordSupported() const
+{
+  return false;
+}
+
+bool MyMoneySqlCipher3Driver::isPasswordSupported() const
+{
+  return true;
+}
+
 //*************************************************
 // replace the QSqlDatabase::tables() call for Mysql only
 // see bug 252841
@@ -638,7 +664,7 @@ const QStringList MyMoneyMysqlDriver::tables(QSql::TableType tt, const QSqlDatab
     case QSql::AllTables:
       selectString = QString("SELECT TABLE_NAME FROM information_schema.TABLES WHERE TABLE_SCHEMA = '%1'").arg(db.databaseName());
       if (!q.exec(selectString)) {
-        throw new MYMONEYEXCEPTION("select names failed in mymoneydbdriver.cpp");
+        throw MYMONEYEXCEPTION("select names failed in mymoneydbdriver.cpp");
       }
       while (q.next())
         tableList.append(q.value(0).toString());
