@@ -33,6 +33,8 @@ private slots:
   void init();
   void cleanup();
   void testArrayOperator(void);
+  void testModifyKey(void);
+  void testModifyKeyTwice(void);
 };
 
 #endif
