@@ -213,6 +213,62 @@ public:
     */
   const QList<MyMoneyPayee> payeeList(void) const;
 
+  /**
+    * This method is used to create a new tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  void addTag(MyMoneyTag& tag);
+
+  /**
+    * This method is used to retrieve information about a tag
+    * An exception will be thrown upon error conditions.
+    *
+    * @param id QString reference to id of tag
+    *
+    * @return MyMoneyTag object of tag
+    */
+  const MyMoneyTag tag(const QString& id) const;
+
+  /**
+    * This method is used to retrieve the id to a corresponding
+    * name of a tag.
+    * An exception will be thrown upon error conditions.
+    *
+    * @param tag QString reference to name of tag
+    *
+    * @return MyMoneyTag reference to object of tag
+    */
+  const MyMoneyTag tagByName(const QString& tag) const;
+
+  /**
+    * This method is used to modify an existing tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  void modifyTag(const MyMoneyTag& tag);
+
+  /**
+    * This method is used to remove an existing tag
+    *
+    * An exception will be thrown upon error conditions
+    *
+    * @param tag MyMoneyTag reference to tag information
+    */
+  void removeTag(const MyMoneyTag& tag);
+
+  /**
+    * This method returns a list of the tags
+    * inside a MyMoneyStorage object
+    *
+    * @return QList<MyMoneyTag> containing the tag information
+    */
+  const QList<MyMoneyTag> tagList(void) const;
+
   /**
     * This method is used to add one account as sub-ordinate to another
     * (parent) account. The objects passed as arguments will be modified
@@ -221,6 +277,7 @@ public:
     * @param parent parent account the account should be added to
     * @param account the account to be added
     */
+
   void addAccount(MyMoneyAccount& parent, MyMoneyAccount& account);
 
   /**
@@ -544,6 +601,7 @@ public:
   virtual void loadTransactions(const QMap<QString, MyMoneyTransaction>& map);
   virtual void loadInstitutions(const QMap<QString, MyMoneyInstitution>& map);
   virtual void loadPayees(const QMap<QString, MyMoneyPayee>& map);
+  virtual void loadTags(const QMap<QString, MyMoneyTag>& map);
   virtual void loadSchedules(const QMap<QString, MyMoneySchedule>& map);
   virtual void loadSecurities(const QMap<QString, MyMoneySecurity>& map);
   virtual void loadCurrencies(const QMap<QString, MyMoneySecurity>& map);
@@ -552,6 +610,7 @@ public:
   virtual void loadAccountId(const unsigned long id);
   virtual void loadTransactionId(const unsigned long id);
   virtual void loadPayeeId(const unsigned long id);
+  virtual void loadTagId(const unsigned long id);
   virtual void loadInstitutionId(const unsigned long id);
   virtual void loadScheduleId(const unsigned long id);
   virtual void loadSecurityId(const unsigned long id);
@@ -567,6 +626,9 @@ public:
   virtual unsigned long payeeId(void) const {
     return m_nextPayeeID;
   };
+  virtual unsigned long tagId(void) const {
+    return m_nextTagID;
+  };
   virtual unsigned long institutionId(void) const {
     return m_nextInstitutionID;
   };
@@ -623,7 +685,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched reference to the MyMoneySchedule object
     */
@@ -633,7 +695,7 @@ public:
     * This method is used to modify an existing MyMoneySchedule
     * object. Therefor, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched const reference to the MyMoneySchedule object to be updated
     */
@@ -643,7 +705,7 @@ public:
     * This method is used to remove an existing MyMoneySchedule object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param sched const reference to the MyMoneySchedule object to be updated
     */
@@ -653,7 +715,7 @@ public:
     * This method is used to retrieve a single MyMoneySchedule object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
@@ -675,7 +737,7 @@ public:
     * This method is used to modify an existing MyMoneySchedule
     * object.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param security reference to the MyMoneySecurity object to be updated
     */
@@ -685,7 +747,7 @@ public:
     * This method is used to remove an existing MyMoneySecurity object
     * from the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param security reference to the MyMoneySecurity object to be removed
     */
@@ -695,7 +757,7 @@ public:
     * This method is used to retrieve a single MyMoneySchedule object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
@@ -714,7 +776,7 @@ public:
     * The ID of the object is the trading symbol, so there is no need for an additional
     * ID since the symbol is guaranteed to be unique.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneyCurrency object
     */
@@ -724,7 +786,7 @@ public:
     * This method is used to modify an existing MyMoneyCurrency
     * object.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneyCurrency object
     */
@@ -734,7 +796,7 @@ public:
     * This method is used to remove an existing MyMoneyCurrency object
     * from the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param currency reference to the MyMoneyCurrency object
     */
@@ -744,7 +806,7 @@ public:
     * This method is used to retrieve a single MyMoneySchedule object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneySchedule object
     * @return MyMoneySchedule object
@@ -755,7 +817,7 @@ public:
     * This method is used to retrieve the list of all currencies
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneyCurrency objects.
     */
@@ -805,7 +867,7 @@ public:
     * This method is used to retrieve the list of all reports
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneyReport objects.
     */
@@ -817,7 +879,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report reference to the MyMoneyReport object
     */
@@ -827,7 +889,7 @@ public:
     * This method is used to load a set of reports into the engine.  This is
     * used when loading from storage, and an ID is already known.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param reports reference to the map of MyMoneyReport objects
     */
@@ -837,7 +899,7 @@ public:
     * This method is used to modify an existing MyMoneyReport
     * object. Therefor, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report const reference to the MyMoneyReport object to be updated
     */
@@ -855,7 +917,7 @@ public:
     * This method is used to retrieve a single MyMoneyReport object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneyReport object
     * @return MyMoneyReport object
@@ -866,7 +928,7 @@ public:
     * This method is used to remove an existing MyMoneyReport object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param report const reference to the MyMoneyReport object to be updated
     */
@@ -876,7 +938,7 @@ public:
     * This method is used to retrieve the list of all budgets
     * known to the engine.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @return QList of all MyMoneyBudget objects.
     */
@@ -888,7 +950,7 @@ public:
     * method returns to the caller, the id will be filled with the
     * newly created object id value.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget reference to the MyMoneyBudget object
     */
@@ -898,7 +960,7 @@ public:
     * This method is used to load a set of budgets into the engine.  This is
     * used when loading from storage, and an ID is already known.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budgets reference to the map of MyMoneyBudget object
     */
@@ -919,7 +981,7 @@ public:
     * This method is used to modify an existing MyMoneyBudget
     * object. Therefore, the id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget const reference to the MyMoneyBudget object to be updated
     */
@@ -937,7 +999,7 @@ public:
     * This method is used to retrieve a single MyMoneyBudget object.
     * The id of the object must be supplied in the parameter @p id.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param id QString containing the id of the MyMoneyBudget object
     * @return MyMoneyBudget object
@@ -948,7 +1010,7 @@ public:
     * This method is used to remove an existing MyMoneyBudget object
     * from the engine. The id attribute of the object must be set.
     *
-    * An exception will be thrown upon erronous situations.
+    * An exception will be thrown upon erroneous situations.
     *
     * @param budget const reference to the MyMoneyBudget object to be updated
     */
@@ -1021,6 +1083,7 @@ private:
   static const int ACCOUNT_ID_SIZE = 6;
   static const int TRANSACTION_ID_SIZE = 18;
   static const int PAYEE_ID_SIZE = 6;
+  static const int TAG_ID_SIZE = 6;
   static const int SCHEDULE_ID_SIZE = 6;
   static const int SECURITY_ID_SIZE = 6;
   static const int REPORT_ID_SIZE = 6;
@@ -1072,6 +1135,13 @@ private:
     */
   unsigned long m_nextPayeeID;
 
+  /**
+    * The member variable m_nextTagID keeps the number that will be
+    * assigned to the next tag created. It is maintained by
+    * nextTagID()
+    */
+  unsigned long m_nextTagID;
+
   /**
     * The member variable m_nextScheduleID keeps the number that will be
     * assigned to the next schedule created. It is maintained by
@@ -1126,6 +1196,11 @@ private:
     */
   MyMoneyMap<QString, MyMoneyPayee> m_payeeList;
 
+  /**
+    * A list containing all the tags that have been used
+    */
+  MyMoneyMap<QString, MyMoneyTag> m_tagList;
+
   /**
     * A list containing all the scheduled transactions
     */
@@ -1206,6 +1281,12 @@ private:
     */
   QString nextPayeeID(void);
 
+  /**
+    * This method is used to get the next valid ID for a tag
+    * @return id for a tag
+    */
+  QString nextTagID(void);
+
   /**
     * This method is used to get the next valid ID for a scheduled transaction
     * @return id for a scheduled transaction
