@@ -273,6 +273,7 @@ void QueryTable::init(void)
       break;
 
     case MyMoneyReport::ePayee:
+    case MyMoneyReport::eTag:
     case MyMoneyReport::eMonth:
     case MyMoneyReport::eWeek:
       constructTransactionTable();
@@ -312,6 +313,9 @@ void QueryTable::init(void)
     case MyMoneyReport::ePayee:
       m_group = "payee";
       break;
+    case MyMoneyReport::eTag:
+      m_group = "tag";
+      break;
     case MyMoneyReport::eMonth:
       m_group = "month";
       break;
@@ -331,7 +335,7 @@ void QueryTable::init(void)
       m_group = "institution,topaccount";
       break;
     default:
-      throw new MYMONEYEXCEPTION("QueryTable::QueryTable(): unhandled row type");
+      throw MYMONEYEXCEPTION("QueryTable::QueryTable(): unhandled row type");
   }
 
   QString sort = m_group + ',' + m_columns + ",id,rank";
@@ -354,6 +358,8 @@ void QueryTable::init(void)
     m_columns += ",number";
   if (qc & MyMoneyReport::eQCpayee)
     m_columns += ",payee";
+  if (qc & MyMoneyReport::eQCtag)
+    m_columns += ",tag";
   if (qc & MyMoneyReport::eQCcategory)
     m_columns += ",category";
   if (qc & MyMoneyReport::eQCaccount)
@@ -396,6 +402,7 @@ void QueryTable::constructTransactionTable(void)
   bool use_transfers;
   bool use_summary;
   bool hide_details;
+  bool tag_special_case = false;
 
   switch (m_config.rowType()) {
     case MyMoneyReport::eCategory:
@@ -409,6 +416,12 @@ void QueryTable::constructTransactionTable(void)
       use_transfers = false;
       hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
       break;
+    case MyMoneyReport::eTag:
+      use_summary = false;
+      use_transfers = false;
+      hide_details = (m_config.detailLevel() == MyMoneyReport::eDetailNone);
+      tag_special_case = true;
+      break;
     default:
       use_summary = true;
       use_transfers = true;
@@ -425,6 +438,7 @@ void QueryTable::constructTransactionTable(void)
 
     TableRow qA, qS;
     QDate pd;
+    QList<QString> tagIdListCache;
 
     qA["id"] = qS["id"] = (* it_transaction).id();
     qA["entrydate"] = qS["entrydate"] = (* it_transaction).entryDate().toString(Qt::ISODate);
@@ -521,15 +535,13 @@ void QueryTable::constructTransactionTable(void)
       QString institution = splitAcc.institutionId();
       QString payee = (*it_split).payeeId();
 
+      const QList<QString> tagIdList = (*it_split).tagIdList();
+
       //convert to base currency
       if (m_config.isConvertCurrency()) {
         xr = (splitAcc.deepCurrencyPrice((*it_transaction).postDate()) * splitAcc.baseCurrencyPrice((*it_transaction).postDate())).reduce();
       } else {
         xr = (splitAcc.deepCurrencyPrice((*it_transaction).postDate())).reduce();
-        //if the currency of the split is different from the currency of the main split, then convert to the currency of the main split
-        if (splitAcc.currency().id() != myBeginCurrency) {
-          xr = (xr * splitAcc.foreignCurrencyPrice(myBeginCurrency, (*it_transaction).postDate())).reduce();
-        }
       }
 
       if (splitAcc.isInvest()) {
@@ -569,12 +581,23 @@ void QueryTable::constructTransactionTable(void)
                       ? i18n("[Empty Payee]")
                       : file->payee(payee).name().simplified();
 
+        if (tag_special_case) {
+          tagIdListCache = tagIdList;
+        } else {
+          QString delimiter = "";
+          for (int i = 0; i < tagIdList.size(); i++) {
+            qA["tag"] += delimiter + file->tag(tagIdList[i]).name().simplified();
+            delimiter = ", ";
+          }
+        }
         qA["reconciledate"] = (*it_split).reconcileDate().toString(Qt::ISODate);
         qA["reconcileflag"] = KMyMoneyUtils::reconcileStateToString((*it_split).reconcileFlag(), true);
         qA["number"] = (*it_split).number();
 
         qA["memo"] = a_memo;
 
+        qA["value"] = (((*it_split).shares()) * xr).convert(fraction).toString();
+
         qS["reconciledate"] = qA["reconciledate"];
         qS["reconcileflag"] = qA["reconcileflag"];
         qS["number"] = qA["number"];
@@ -666,13 +689,27 @@ void QueryTable::constructTransactionTable(void)
 
               qA["split"] = "";
 
-              //multiply by currency and convert to lowest fraction
-              qA["value"] = ((-(*it_split).shares()) * xr).convert(fraction).toString();
+              // multiply by currency and convert to lowest fraction
+              // but only for income and expense
+              // transfers are dealt with somewhere else below
+              if (splitAcc.isIncomeExpense()) {
+                qA["value"] = ((-(*it_split).shares()) * xr).convert(fraction).toString();
+              }
               qA["rank"] = '0';
             }
 
             qA ["memo"] = (*it_split).memo();
 
+            // if different from base currency and not converting
+            // show the currency of the split
+            if (splitAcc.currencyId() != file->baseCurrency().id()) {
+              if (!report.isConvertCurrency()) {
+                qS["currency"] = splitAcc.currencyId();
+              }
+            } else {
+              qS["currency"] = "";
+            }
+
             if (! splitAcc.isIncomeExpense()) {
               qA["category"] = ((*it_split).shares().isNegative()) ?
                                i18n("Transfer from %1", splitAcc.fullName())
@@ -696,7 +733,17 @@ void QueryTable::constructTransactionTable(void)
                   || (!m_config.isInvertingText()
                       && (transaction_text
                           || m_config.match(&(*it_split))))) {
-                m_rows += qA;
+                if (tag_special_case) {
+                  if (!tagIdListCache.size())
+                    qA["tag"] = i18n("[No Tag]");
+                  else
+                    for (int i = 0; i < tagIdListCache.size(); i++) {
+                      qA["tag"] = file->tag(tagIdListCache[i]).name().simplified();
+                      m_rows += qA;
+                    }
+                } else {
+                  m_rows += qA;
+                }
               }
             }
           }
@@ -704,7 +751,6 @@ void QueryTable::constructTransactionTable(void)
 
         if (m_config.includes(splitAcc) && use_transfers) {
           if (! splitAcc.isIncomeExpense()) {
-
             //multiply by currency and convert to lowest fraction
             qS["value"] = ((*it_split).shares() * xr).convert(fraction).toString();
 
@@ -726,6 +772,13 @@ void QueryTable::constructTransactionTable(void)
                          ? a_memo
                          : (*it_split).memo();
 
+            //FIXME-ALEX When is used this? I can't find in which condition we arrive here... maybe this code is useless?
+            QString delimiter = "";
+            for (int i = 0; i < tagIdList.size(); i++) {
+              qA["tag"] += delimiter + file->tag(tagIdList[i]).name().simplified();
+              delimiter = '+';
+            }
+
             qS["payee"] = payee.isEmpty()
                           ? qA["payee"]
                           : file->payee(payee).name().simplified();
@@ -765,7 +818,6 @@ void QueryTable::constructTransactionTable(void)
         break;
 
     } while (it_split != myBegin);
-
     if (loan_special_case) {
       m_rows += qA;
     }
@@ -1242,14 +1294,13 @@ void QueryTable::constructSplitsTable(void)
     // transaction only has income and expense splits (which should not happen). In that case, point
     // it to the first split
     if (myBegin == splits.end()) {
-        myBegin = splits.begin();
+      myBegin = splits.begin();
     }
 
     //the account of the beginning splits
     ReportAccount myBeginAcc = (*myBegin).accountId();
 
     bool include_me = true;
-    bool transaction_text = false; //indicates whether a text should be considered as a match for the transaction or for a split only
     QString a_fullname = "";
     QString a_memo = "";
     int pass = 1;
@@ -1268,6 +1319,8 @@ void QueryTable::constructSplitsTable(void)
       QString institution = splitAcc.institutionId();
       QString payee = (*it_split).payeeId();
 
+      const QList<QString> tagIdList = (*it_split).tagIdList();
+
       if (m_config.isConvertCurrency()) {
         xr = (splitAcc.deepCurrencyPrice((*it_transaction).postDate()) * splitAcc.baseCurrencyPrice((*it_transaction).postDate())).reduce();
       } else {
@@ -1276,7 +1329,7 @@ void QueryTable::constructSplitsTable(void)
 
       // reverse the sign of incomes and expenses to keep consistency in the way it is displayed in other reports
       if (splitAcc.isIncomeExpense()) {
-          xr = -xr;
+        xr = -xr;
       }
 
       if (splitAcc.isInvest()) {
@@ -1299,8 +1352,6 @@ void QueryTable::constructSplitsTable(void)
       a_fullname = splitAcc.fullName();
       a_memo = (*it_split).memo();
 
-      transaction_text = m_config.match(&(*it_split));
-
       qA["price"] = xr.convert(MyMoneyMoney::precToDenom(KMyMoneyGlobalSettings::pricePrecision())).toString();
       qA["account"] = splitAcc.name();
       qA["accountid"] = splitAcc.id();
@@ -1310,6 +1361,13 @@ void QueryTable::constructSplitsTable(void)
                           ? i18n("No Institution")
                           : file->institution(institution).name();
 
+      //FIXME-ALEX Is this useless? Isn't constructSplitsTable called only for cashflow type report?
+      QString delimiter = "";
+      for (int i = 0; i < tagIdList.size(); i++) {
+        qA["tag"] += delimiter + file->tag(tagIdList[i]).name().simplified();
+        delimiter = ',';
+      }
+
       qA["payee"] = payee.isEmpty()
                     ? i18n("[Empty Payee]")
                     : file->payee(payee).name().simplified();
@@ -1494,4 +1552,3 @@ void QueryTable::constructSplitsTable(void)
 
 
 }
-// vim:cin:si:ai:et:ts=2:sw=2:
