@@ -272,9 +272,8 @@ void QueryTableTest::testQueryBasics()
     QVERIFY(searchHTML(html, i18nc("Total balance", "Total") + " Week of 2004-11-01") == -(moChild) * 3);
     QVERIFY(searchHTML(html, i18nc("Total balance", "Total") + " Week of 2005-08-29") == -moParent2);
     QVERIFY(searchHTML(html, i18nc("Grand total balance", "Grand Total")) == -(moParent1 + moParent2 + moSolo + moChild) * 3 + moCheckingOpen + moCreditOpen);
-  } catch (MyMoneyException *e) {
-    QFAIL(qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
   }
 
   // Test querytable::TableRow::operator> and operator==
@@ -415,9 +414,8 @@ void QueryTableTest::testAccountQuery()
     QVERIFY(searchHTML(html, i18nc("Total balance", "Total") + ' ' + i18n("Checking")) == moCheckingOpen - moSolo*3);
     QVERIFY(searchHTML(html, i18nc("Total balance", "Total") + ' ' + i18n("Credit Card")) == moCreditOpen - (moParent1 + moParent2 + moChild) * 3);
     QVERIFY(searchHTML(html, i18nc("Grand total balance", "Grand Total")) == moCheckingOpen + moCreditOpen - (moParent1 + moParent2 + moSolo + moChild) * 3);
-  } catch (MyMoneyException *e) {
-    QFAIL(qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
   }
 }
 
@@ -562,7 +560,7 @@ void QueryTableTest::testInvestment(void)
 
     rows = invhold.rows();
 
-    QVERIFY(rows.count() == 2);
+    QVERIFY(rows.count() == 1);
     QVERIFY(MyMoneyMoney(rows[0]["return"]) == MyMoneyMoney("669/10000"));
     QVERIFY(MyMoneyMoney(rows[0]["buys"]) == MyMoneyMoney(210000.00));
     QVERIFY(MyMoneyMoney(rows[0]["sells"]) == MyMoneyMoney(-44000.00));
@@ -570,7 +568,6 @@ void QueryTableTest::testInvestment(void)
     QVERIFY(MyMoneyMoney(rows[0]["cashincome"]) == MyMoneyMoney(3300.00));
     QVERIFY(MyMoneyMoney(rows[0]["shares"]) == MyMoneyMoney(1700.00));
     QVERIFY(MyMoneyMoney(rows[0]["price"]) == MyMoneyMoney(100.00));
-    QVERIFY(MyMoneyMoney(rows[1]["return"]).isZero());
 
     html = invhold.renderBody();
     QVERIFY(searchHTML(html, i18n("Grand Total")) == MyMoneyMoney(170000.00));
@@ -588,9 +585,8 @@ void QueryTableTest::testInvestment(void)
     invhold.dump("invhold.html", "<html><head></head><body>%1</body></html>");
 #endif
 
-  } catch (MyMoneyException *e) {
-    QFAIL(qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
   }
 }
 //this is to prevent me from making mistakes again when modifying balances - asoliverez
@@ -639,9 +635,91 @@ void QueryTableTest::testBalanceColumn()
     QVERIFY(html.indexOf(closingDate + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Closing Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>&nbsp;-702.36</td></tr>") > 0);
     QVERIFY(html.indexOf(closingDate + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Closing Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>&nbsp;-705.69</td></tr>") > 0);
 
-  } catch (MyMoneyException *e) {
-    QFAIL(qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
+  }
+
+}
+
+void QueryTableTest::testBalanceColumnWithMultipleCurrencies()
+{
+  try {
+
+    MyMoneyMoney moJpyOpening(0.0, 1);
+    MyMoneyMoney moJpyPrice(0.010, 100);
+    MyMoneyMoney moJpyPrice2(0.011, 100);
+    MyMoneyMoney moJpyPrice3(0.024, 100);
+    MyMoneyMoney moTransaction(100, 1);
+    MyMoneyMoney moJpyTransaction(100, 1);
+
+    QString acJpyChecking = makeAccount(QString("Japanese Checking"), MyMoneyAccount::Checkings, moJpyOpening, QDate(2003, 11, 15), acAsset, "JPY");
+
+    makePrice("JPY", QDate(2004, 1, 1), MyMoneyMoney(moJpyPrice));
+    makePrice("JPY", QDate(2004, 5, 1), MyMoneyMoney(moJpyPrice2));
+    makePrice("JPY", QDate(2004, 6, 30), MyMoneyMoney(moJpyPrice3));
+
+    QDate openingDate(2004, 2, 20);
+    QDate intermediateDate(2004, 5, 20);
+    QDate closingDate(2004, 7, 20);
+
+    TransactionHelper t1(openingDate,      MyMoneySplit::ActionTransfer,   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
+    TransactionHelper t4(openingDate,      MyMoneySplit::ActionDeposit,    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
+    TransactionHelper t2(intermediateDate, MyMoneySplit::ActionTransfer,   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
+    TransactionHelper t5(intermediateDate, MyMoneySplit::ActionDeposit,    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
+    TransactionHelper t3(closingDate,      MyMoneySplit::ActionTransfer,   MyMoneyMoney(moJpyTransaction), acJpyChecking, acChecking, "JPY");
+    TransactionHelper t6(closingDate,      MyMoneySplit::ActionDeposit,    MyMoneyMoney(moTransaction),    acCredit,      acChecking);
+
+    unsigned cols;
+
+    MyMoneyReport filter;
+
+    filter.setRowType(MyMoneyReport::eAccount);
+    filter.setName("Transactions by Account");
+    cols = MyMoneyReport::eQCnumber | MyMoneyReport::eQCpayee | MyMoneyReport::eQCcategory | MyMoneyReport::eQCbalance;
+    filter.setQueryColumns(static_cast<MyMoneyReport::EQueryColumns>(cols));
+    // don't convert values to the default currency
+    filter.setConvertCurrency(false);
+    XMLandback(filter);
+    QueryTable qtbl_3(filter);
+
+    writeTabletoHTML(qtbl_3, "Transactions by Account (multiple currencies).html");
+
+    QString html = qtbl_3.renderBody();
+
+    QList<ListTable::TableRow> rows = qtbl_3.rows();
+
+    QVERIFY(rows.count() == 18);
+
+    //this is to make sure that the dates of closing and opening balances and the balance numbers are ok
+    QString openingDateString = KGlobal::locale()->formatDate(openingDate, KLocale::ShortDate);
+    QString intermediateDateString = KGlobal::locale()->formatDate(intermediateDate, KLocale::ShortDate);
+    QString closingDateString = KGlobal::locale()->formatDate(closingDate, KLocale::ShortDate);
+    // check the opening and closing balances
+    QVERIFY(html.indexOf(openingDateString + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Opening Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>&nbsp;0.00</td></tr>") > 0);
+    QVERIFY(html.indexOf(closingDateString + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Closing Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>&nbsp;304.00</td></tr>") > 0);
+    QVERIFY(html.indexOf(closingDateString + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Closing Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>&nbsp;-300.00</td></tr>") > 0);
+    QVERIFY(html.indexOf(closingDateString + "</td><td class=\"left\"></td><td class=\"left\">" + i18n("Closing Balance") + "</td><td class=\"left\"></td><td class=\"value\"></td><td>JPY&nbsp;-300.00</td></tr>") > 0);
+
+    // after a transfer of 100 JPY the balance should be 1.00 - price is 0.010 (precision of 2)
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000001>" + openingDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Japanese Checking</td><td class=\"value\">&nbsp;1.00</td><td>&nbsp;1.00</td></tr>") > 0);
+
+    // after a transfer of 100 the balance should be 101.00
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000002>" + openingDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Credit Card</td><td class=\"value\">&nbsp;100.00</td><td>&nbsp;101.00</td></tr>") > 0);
+
+    // after a transfer of 100 JPY the balance should be 102.00 - price is 0.011 (precision of 2)
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000003>" + intermediateDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Japanese Checking</td><td class=\"value\">&nbsp;1.00</td><td>&nbsp;102.00</td></tr>") > 0);
+
+    // after a transfer of 100 the balance should be 202.00
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000004>" + intermediateDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Credit Card</td><td class=\"value\">&nbsp;100.00</td><td>&nbsp;202.00</td></tr>") > 0);
+
+    // after a transfer of 100 JPY the balance should be 204.00 - price is 0.024 (precision of 2)
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000005>" + closingDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Japanese Checking</td><td class=\"value\">&nbsp;2.00</td><td>&nbsp;204.00</td></tr>") > 0);
+
+    // after a transfer of 100 the balance should be 304.00
+    QVERIFY(html.indexOf("<a href=ledger?id=A000001&tid=T000000000000000006>" + closingDateString + "</a></td><td class=\"left\"></td><td class=\"left\">Test Payee</td><td class=\"left\">Transfer from Credit Card</td><td class=\"value\">&nbsp;100.00</td><td>&nbsp;304.00</td></tr>") > 0);
+
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
   }
 
 }
@@ -670,11 +748,7 @@ void QueryTableTest::testTaxReport()
 
     QString html = qtbl_3.renderBody();
     QVERIFY(rows.count() == 1);
-  } catch (MyMoneyException *e) {
-    QFAIL(qPrintable(e->what()));
-    delete e;
+  } catch (const MyMoneyException &e) {
+    QFAIL(qPrintable(e.what()));
   }
 }
-
-#include "querytabletest.moc"
-// vim:cin:si:ai:et:ts=2:sw=2:
