@@ -41,9 +41,17 @@
 #include "mymoneyforecast.h"
 #include "pivottable.h"
 #include "pivotgrid.h"
+#include "fixedcolumntreeview.h"
 
 KForecastView::KForecastView(QWidget *parent) :
-    QWidget(parent)
+    QWidget(parent),
+    m_totalItem(0),
+    m_assetItem(0),
+    m_liabilityItem(0),
+    m_incomeItem(0),
+    m_expenseItem(0),
+    m_chartLayout(0),
+    m_forecastChart(0)
 {
   setupUi(this);
 
@@ -56,14 +64,15 @@ KForecastView::KForecastView(QWidget *parent) :
   KConfigGroup grp = config->group("Last Use Settings");
   m_tab->setCurrentIndex(grp.readEntry("KForecastView_LastType", 0));
 
-  m_forecastButton->setIcon(KIcon("forecast"));
+  m_forecastButton->setIcon(KIcon("view-financial-forecast"));
 
-  connect(m_tab, SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabChanged(QWidget*)));
+  connect(m_tab, SIGNAL(currentChanged(int)), this, SLOT(slotTabChanged(int)));
 
   connect(MyMoneyFile::instance(), SIGNAL(dataChanged()), this, SLOT(slotLoadForecast()));
 
   connect(m_forecastButton, SIGNAL(clicked()), this, SLOT(slotManualForecast()));
 
+  m_forecastList->setUniformRowHeights(true);
   m_forecastList->setAllColumnsShowFocus(true);
   m_summaryList->setAllColumnsShowFocus(true);
   m_budgetList->setAllColumnsShowFocus(true);
@@ -88,9 +97,9 @@ KForecastView::~KForecastView()
 {
 }
 
-void KForecastView::slotTabChanged(QWidget* _tab)
+void KForecastView::slotTabChanged(int index)
 {
-  ForecastViewTab tab = static_cast<ForecastViewTab>(m_tab->indexOf(_tab));
+  ForecastViewTab tab = static_cast<ForecastViewTab>(index);
 
   // remember this setting for startup
   KSharedConfigPtr config = KGlobal::config();
@@ -131,7 +140,7 @@ void KForecastView::showEvent(QShowEvent* event)
 {
   emit aboutToShow();
 
-  slotTabChanged(m_tab->currentWidget());
+  slotTabChanged(m_tab->currentIndex());
 
   // don't forget base class implementation
   QWidget::showEvent(event);
@@ -149,7 +158,7 @@ void KForecastView::slotLoadForecast(void)
   loadForecastSettings();
 
   if (isVisible())
-    slotTabChanged(m_tab->currentWidget());
+    slotTabChanged(m_tab->currentIndex());
 }
 
 void KForecastView::slotManualForecast(void)
@@ -161,7 +170,7 @@ void KForecastView::slotManualForecast(void)
   m_needReload[ChartView] = true;
 
   if (isVisible())
-    slotTabChanged(m_tab->currentWidget());
+    slotTabChanged(m_tab->currentIndex());
 }
 
 void KForecastView::loadForecastSettings(void)
@@ -206,6 +215,7 @@ void KForecastView::loadListView(void)
   forecast.doForecast();
 
   m_forecastList->clear();
+  m_forecastList->setColumnCount(0);
   m_forecastList->setIconSize(QSize(22, 22));
   m_forecastList->setSortingEnabled(true);
   m_forecastList->sortByColumn(0, Qt::AscendingOrder);
@@ -237,6 +247,9 @@ void KForecastView::loadListView(void)
   loadAccounts(forecast, file->liability(), m_liabilityItem, eDetailed);
 
   adjustHeadersAndResizeToContents(m_forecastList);
+
+  // add the fixed column only if the horizontal scroll bar is visible
+  m_fixedColumnView.reset(m_forecastList->horizontalScrollBar()->isVisible() ? new FixedColumnTreeView(m_forecastList) : 0);
 }
 
 void KForecastView::loadSummaryView(void)
@@ -444,6 +457,7 @@ void KForecastView::loadAdvancedView(void)
   //clear the list, including columns
   m_advancedList->clear();
   m_advancedList->setColumnCount(0);
+  m_advancedList->setIconSize(QSize(22, 22));
 
   QStringList headerLabels;
 
@@ -490,6 +504,7 @@ void KForecastView::loadAdvancedView(void)
 
     advancedItem = new QTreeWidgetItem(m_advancedList, advancedItem, false);
     advancedItem->setText(0, acc.name());
+    advancedItem->setIcon(0, acc.accountPixmap());
     int it_c = 1; // iterator for the columns of the listview
 
     //get minimum balance list
@@ -626,9 +641,8 @@ QList<MyMoneyPrice> KForecastView::getAccountPrices(const MyMoneyAccount& acc)
       }
     }
 
-  } catch (MyMoneyException *e) {
-    kDebug(2) << Q_FUNC_INFO << " caught exception while adding " << acc.name() << "[" << acc.id() << "]: " << e->what();
-    delete e;
+  } catch (const MyMoneyException &e) {
+    kDebug(2) << Q_FUNC_INFO << " caught exception while adding " << acc.name() << "[" << acc.id() << "]: " << e.what();
   }
   return prices;
 }
@@ -708,10 +722,15 @@ bool KForecastView::includeAccount(MyMoneyForecast& forecast, const MyMoneyAccou
 
 void KForecastView::adjustHeadersAndResizeToContents(QTreeWidget *widget)
 {
+  QSize sizeHint(0, widget->sizeHintForRow(0));
   QTreeWidgetItem *header = widget->headerItem();
   for (int i = 0; i < header->columnCount(); ++i) {
-    if (i && header) {
+    if (i > 0) {
       header->setData(i, Qt::TextAlignmentRole, Qt::AlignRight);
+      // make sure that the row height stays the same even when the column that has icons is not visible
+      if (m_totalItem) {
+        m_totalItem->setSizeHint(i, sizeHint);
+      }
     }
     widget->resizeColumnToContents(i);
   }
@@ -1016,5 +1035,3 @@ void KForecastView::loadChartView(void)
   m_forecastChart->update();
 
 }
-
-#include "kforecastview.moc"
