@@ -34,7 +34,8 @@
 #include <kcombobox.h>
 #include <klineedit.h>
 #include <kurlrequester.h>
-
+#include <kmymoneyedit.h>
+#include "kmymoneymvccombo.h"
 // ----------------------------------------------------------------------------
 // Project Includes
 
@@ -49,6 +50,7 @@
  * Enhanced by Thomas Baumgart to support the lineedit field of a         *
  * a KComboBox.                                                           *
  *                                                                        *
+ * With further widgets added by Allan Anderson for missing fields.       *
  **************************************************************************/
 
 void kMandatoryFieldGroup::add(QWidget *widget)
@@ -69,10 +71,21 @@ void kMandatoryFieldGroup::add(QWidget *widget)
       }
     }
 
-    else if (qobject_cast<QLineEdit*>(widget))
+    else if (qobject_cast<kMyMoneyEdit*>(widget)) {
+      kMyMoneyEdit* amount = qobject_cast<kMyMoneyEdit*>(widget);
+      KLineEdit* lineedit = qobject_cast<KLineEdit*>(amount->lineedit());
+      if (lineedit) {
+        connect(lineedit, SIGNAL(textChanged(QString)), this, SLOT(changed()));
+      } else {
+        connect(amount, SIGNAL(highlighted(int)), this, SLOT(changed()));
+      }
+    }
+
+    else if (qobject_cast<QLineEdit*>(widget)) {
       connect(qobject_cast<QLineEdit*>(widget),
               SIGNAL(textChanged(QString)),
               this, SLOT(changed()));
+    }
 
     else if (qobject_cast<QSpinBox*>(widget))
       connect(qobject_cast<QSpinBox*>(widget),
@@ -102,7 +115,6 @@ void kMandatoryFieldGroup::add(QWidget *widget)
   }
 }
 
-
 void kMandatoryFieldGroup::remove(QWidget *widget)
 {
   widget->setPalette(QApplication::palette());
@@ -110,7 +122,6 @@ void kMandatoryFieldGroup::remove(QWidget *widget)
   changed();
 }
 
-
 void kMandatoryFieldGroup::setOkButton(QPushButton *button)
 {
   if (m_okButton && m_okButton != button)
@@ -119,7 +130,6 @@ void kMandatoryFieldGroup::setOkButton(QPushButton *button)
   changed();
 }
 
-
 void kMandatoryFieldGroup::changed(void)
 {
   bool enable = true;
@@ -130,6 +140,13 @@ void kMandatoryFieldGroup::changed(void)
     if (!(widget->isEnabled())) {
       continue;
     }
+    if (qobject_cast<KMyMoneyPayeeCombo*>(widget)) {
+      if ((dynamic_cast<KMyMoneyPayeeCombo*>(widget))->lineEdit()->text().isEmpty()) {
+        enable = false;
+        break;
+      } else
+        continue;
+    }
     if (qobject_cast<QCheckBox*>(widget)) {
       if ((qobject_cast<QCheckBox*>(widget))->checkState() == Qt::PartiallyChecked) {
         enable = false;
@@ -165,6 +182,13 @@ void kMandatoryFieldGroup::changed(void)
       } else
         continue;
     }
+    if ((qobject_cast<kMyMoneyEdit*>(widget))) {
+      if ((qobject_cast<kMyMoneyEdit*>(widget))->text() == "0/1") {
+        enable = false;
+        break;
+      } else
+        continue;
+    }
   }
 
   if (m_okButton)
@@ -175,7 +199,6 @@ void kMandatoryFieldGroup::changed(void)
   emit stateChanged(enable);
 }
 
-
 void kMandatoryFieldGroup::clear(void)
 {
   QList<QWidget *>::Iterator i;
@@ -188,7 +211,3 @@ void kMandatoryFieldGroup::clear(void)
     m_enabled = true;
   }
 }
-
-
-#include "kguiutils.moc"
-
