@@ -25,13 +25,19 @@
 #include <QStandardItem>
 #include <QAbstractItemView>
 #include <QSortFilterProxyModel>
+#include <QHBoxLayout>
+#include <QFrame>
+#include <QLabel>
+#include <QToolButton>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
-
+#include <kapplication.h>
 #include <klocale.h>
 #include <kdebug.h>
 #include <klineedit.h>
+#include <KIconLoader>
+#include <kmessagebox.h>
 
 // ----------------------------------------------------------------------------
 // Project Includes
@@ -129,6 +135,16 @@ void KMyMoneyMVCCombo::setSubstringSearch(bool enabled)
   }
 }
 
+void KMyMoneyMVCCombo::setSubstringSearchForChildren(QWidget*const widget, bool enabled)
+{
+  Q_CHECK_PTR(widget);
+  QList<KMyMoneyMVCCombo *> comboList;
+  comboList = widget->findChildren<KMyMoneyMVCCombo *>();
+  foreach (KMyMoneyMVCCombo *combo, comboList) {
+    combo->setSubstringSearch(enabled);
+  }
+}
+
 void KMyMoneyMVCCombo::setModel(QAbstractItemModel *model)
 {
   if (!model)
@@ -215,6 +231,21 @@ void KMyMoneyMVCCombo::focusOutEvent(QFocusEvent* e)
     return;
   }
 
+  //check if the focus went to a widget in TransactionFrom or in the Register
+  if (e->reason() == Qt::MouseFocusReason) {
+    QObject *w = this->parent();
+    QObject *q = qApp->focusWidget()->parent();
+    // KMyMoneyTagCombo is inside KTagContainer, KMyMoneyPayeeCombo isn't it
+    if (w->inherits("KTagContainer"))
+      w = w->parent();
+    while (q && q->objectName() != "qt_scrollarea_viewport")
+      q = q->parent();
+    if (q != w && qApp->focusWidget()->objectName() != "register") {
+      KComboBox::focusOutEvent(e);
+      return;
+    }
+  }
+
   d->m_inFocusOutEvent = true;
   if (isEditable() && !currentText().isEmpty() && e->reason() != Qt::ActiveWindowFocusReason) {
     if (d->m_canCreateObjects) {
@@ -229,21 +260,8 @@ void KMyMoneyMVCCombo::focusOutEvent(QFocusEvent* e)
         }
       }
 
-      if (!contains(currentText())) {
-        QString id;
-        // annouce that we go into a possible dialog to create an object
-        // This can be used by upstream widgets to disable filters etc.
-        emit objectCreation(true);
-
-        emit createItem(currentText(), id);
-
-        // Announce that we return from object creation
-        emit objectCreation(false);
-
-        // update the field to a possibly created object
-        m_id = id;
-        setCurrentTextById(id);
-      }
+      //check if the current text is contained in the internal list, if not ask the user if want to create a new item.
+      checkCurrentText();
 
       // else if we cannot create objects, and the current text is not
       // in the list, then we clear the text and the selection.
@@ -269,6 +287,27 @@ void KMyMoneyMVCCombo::focusOutEvent(QFocusEvent* e)
   }
 
   d->m_inFocusOutEvent = false;
+  // This is used only be KMyMoneyTagCombo at this time
+  emit lostFocus();
+}
+
+void KMyMoneyMVCCombo::checkCurrentText(void)
+{
+  if (!contains(currentText())) {
+    QString id;
+    // annouce that we go into a possible dialog to create an object
+    // This can be used by upstream widgets to disable filters etc.
+    emit objectCreation(true);
+
+    emit createItem(currentText(), id);
+
+    // Announce that we return from object creation
+    emit objectCreation(false);
+
+    // update the field to a possibly created object
+    m_id = id;
+    setCurrentTextById(id);
+  }
 }
 
 void KMyMoneyMVCCombo::setCurrentTextById(const QString& id)
@@ -317,6 +356,168 @@ void KMyMoneyPayeeCombo::loadPayees(const QList<MyMoneyPayee>& list)
   clearEditText();
 }
 
+KMyMoneyTagCombo::KMyMoneyTagCombo(QWidget* parent) :
+    KMyMoneyMVCCombo(true, parent)
+{
+}
+
+void KMyMoneyTagCombo::loadTags(const QList<MyMoneyTag>& list)
+{
+  clear();
+
+  //add a blank item, since the field is optional
+  addItem(QString(), QVariant(QString()));
+
+  //add all not closed tags
+  QList<MyMoneyTag>::const_iterator it;
+  for (it = list.constBegin(); it != list.constEnd(); ++it) {
+    if (!(*it).isClosed())
+      addItem((*it).name(), QVariant((*it).id()));
+    else {
+      m_closedIdList.append((*it).id());
+      m_closedTagNameList.append((*it).name());
+    }
+  }
+
+  //sort the model, which will sort the list in the combo
+  model()->sort(Qt::DisplayRole, Qt::AscendingOrder);
+
+  //set the text to empty and the index to the first item on the list
+  setCurrentIndex(0);
+  clearEditText();
+}
+
+void KMyMoneyTagCombo::setUsedTagList(QList<QString>& usedIdList, QList<QString>& usedTagNameList)
+{
+  m_usedIdList = usedIdList;
+  m_usedTagNameList = usedTagNameList;
+  for (int i = 0; i < m_usedIdList.size(); ++i) {
+    int index = findData(QVariant(m_usedIdList.at(i)), Qt::UserRole, Qt::MatchExactly);
+    if (index != -1) removeItem(index);
+  }
+}
+
+void KMyMoneyTagCombo::checkCurrentText(void)
+{
+  if (!contains(currentText())) {
+    if (m_closedTagNameList.contains(currentText())) {
+      // Tell the user what's happened
+      QString msg = QString("<qt>") + i18n("Closed tags can't be used.") + QString("</qt>");
+      KMessageBox::information(this, msg, i18n("Closed tag"), "Closed tag");
+      setCurrentText();
+      return;
+    } else if (m_usedTagNameList.contains(currentText())) {
+      // Tell the user what's happened
+      QString msg = QString("<qt>") + i18n("The tag is already present.") + QString("</qt>");
+      KMessageBox::information(this, msg, i18n("Duplicate tag"), "Duplicate tag");
+      setCurrentText();
+      return;
+    }
+    QString id;
+    // annouce that we go into a possible dialog to create an object
+    // This can be used by upstream widgets to disable filters etc.
+    emit objectCreation(true);
+
+    emit createItem(currentText(), id);
+
+    // Announce that we return from object creation
+    emit objectCreation(false);
+
+    // update the field to a possibly created object
+    //m_id = id;
+    setCurrentTextById(id);
+  }
+}
+
+KTagLabel::KTagLabel(const QString& id, const QString& name, QWidget* parent) :
+    QFrame(parent)
+{
+  QToolButton *t = new QToolButton(this);
+  t->setIcon(QIcon(SmallIcon("dialog-close")));
+  t->setAutoRaise(true);
+  QLabel *l = new QLabel(name, this);
+  m_tagId = id;
+  QHBoxLayout *layout = new QHBoxLayout;
+  layout->setContentsMargins(0, 0, 0, 0);
+  layout->setSpacing(0);
+  this->setLayout(layout);
+  layout->addWidget(t);
+  layout->addWidget(l);
+  connect(t, SIGNAL(clicked(bool)), this, SIGNAL(clicked(bool)));
+  //this->setFrameStyle(QFrame::Panel | QFrame::Plain);
+}
+
+KTagContainer::KTagContainer(QWidget* parent) :
+    QWidget(parent)
+{
+  m_tagCombo = new KMyMoneyTagCombo;
+  QHBoxLayout *layout = new QHBoxLayout;
+  layout->setContentsMargins(0, 0, 5, 0);
+  layout->setSpacing(0);
+  layout->addWidget(m_tagCombo, 100);
+  this->setLayout(layout);
+  this->setFocusProxy(m_tagCombo);
+  connect(m_tagCombo, SIGNAL(lostFocus()), this, SLOT(slotAddTagWidget()));
+}
+
+void KTagContainer::loadTags(const QList<MyMoneyTag>& list)
+{
+  m_list = list;
+  m_tagCombo->loadTags(list);
+}
+
+const QList<QString> KTagContainer::selectedTags(void)
+{
+  return m_tagIdList;
+}
+
+void KTagContainer::addTagWidget(const QString& id)
+{
+  if (id.isNull() || m_tagIdList.contains(id))
+    return;
+  const QString tagName = m_tagCombo->itemText(m_tagCombo->findData(QVariant(id), Qt::UserRole, Qt::MatchExactly));
+  KTagLabel *t = new KTagLabel(id, tagName, this);
+  connect(t, SIGNAL(clicked(bool)), this, SLOT(slotRemoveTagWidget()));
+  m_tagLabelList.append(t);
+  m_tagNameList.append(tagName);
+  m_tagIdList.append(id);
+  this->layout()->addWidget(t);
+  m_tagCombo->loadTags(m_list);
+  m_tagCombo->setUsedTagList(m_tagIdList, m_tagNameList);
+  m_tagCombo->setCurrentIndex(0);
+  m_tagCombo->setFocus();
+}
+
+void KTagContainer::RemoveAllTagWidgets(void)
+{
+  m_tagIdList.clear();
+  m_tagNameList.clear();
+  while (!m_tagLabelList.isEmpty())
+    delete m_tagLabelList.takeLast();
+  m_tagCombo->loadTags(m_list);
+  m_tagCombo->setUsedTagList(m_tagIdList, m_tagNameList);
+  m_tagCombo->setCurrentIndex(0);
+}
+
+void KTagContainer::slotAddTagWidget(void)
+{
+  addTagWidget(m_tagCombo->selectedItem());
+}
+
+void KTagContainer::slotRemoveTagWidget(void)
+{
+  this->tagCombo()->setFocus();
+  KTagLabel *t = (KTagLabel *)sender();
+  int index = m_tagLabelList.indexOf(t);
+  m_tagLabelList.removeAt(index);
+  m_tagIdList.removeAt(index);
+  m_tagNameList.removeAt(index);
+  delete t;
+  m_tagCombo->loadTags(m_list);
+  m_tagCombo->setUsedTagList(m_tagIdList, m_tagNameList);
+  m_tagCombo->setCurrentIndex(0);
+}
+
 KMyMoneyReconcileCombo::KMyMoneyReconcileCombo(QWidget* w) :
     KMyMoneyMVCCombo(false, w)
 {
@@ -541,6 +742,7 @@ KMyMoneyPeriodCombo::KMyMoneyPeriodCombo(QWidget* parent) :
   insertItem(i18n("Next quarter"), MyMoneyTransactionFilter::nextQuarter);
   insertItem(i18n("Next 6 months"), MyMoneyTransactionFilter::next6Months);
   insertItem(i18n("Next 12 months"), MyMoneyTransactionFilter::next12Months);
+  insertItem(i18n("Next 18 months"), MyMoneyTransactionFilter::next18Months);
   insertItem(i18n("Last 3 months to next 3 months"), MyMoneyTransactionFilter::last3ToNext3Months);
   insertItem(i18n("User defined"), MyMoneyTransactionFilter::userDefined);
 }
@@ -647,4 +849,4 @@ void KMyMoneyFrequencyCombo::slotCurrentDataChanged()
   emit currentDataChanged(currentData());
 }
 
-#include "kmymoneymvccombo.moc"
+#include "moc_kmymoneymvccombo.cpp"
\ No newline at end of file
