@@ -28,13 +28,14 @@
 #include <QBoxLayout>
 #include <QHeaderView>
 #include <QApplication>
+#include <QTextDocument>
+#include <QAbstractTextDocumentLayout>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
 #include <klocale.h>
 #include <kglobal.h>
-#include <kiconloader.h>
 #include <kdebug.h>
 #include <kpushbutton.h>
 
@@ -45,6 +46,7 @@
 #include <mymoneysplit.h>
 #include <mymoneyfile.h>
 #include <mymoneypayee.h>
+#include <mymoneytag.h>
 #include <register.h>
 #include <kmymoneycategory.h>
 #include <kmymoneydateinput.h>
@@ -152,7 +154,7 @@ Transaction::Transaction(Register *parent, const MyMoneyTransaction& transaction
     m_formRowHeight(-1),
     m_selected(false),
     m_focus(false),
-    m_erronous(false),
+    m_erroneous(false),
     m_inEdit(false),
     m_inRegisterEdit(false),
     m_showBalance(true),
@@ -174,12 +176,21 @@ Transaction::Transaction(Register *parent, const MyMoneyTransaction& transaction
     m_payeeHeader = m_split.shares().isNegative() ? i18n("Pay to") : i18n("From");
   }
 
+  // load the tag
+  if (!m_split.tagIdList().isEmpty()) {
+    const QList<QString> t = m_split.tagIdList();
+    for (int i = 0; i < t.count(); i++) {
+      m_tagList << file->tag(t[i]).name();
+      m_tagColorList << file->tag(t[i]).tagColor();
+    }
+  }
+
   // load the currency
   if (!m_transaction.id().isEmpty())
     m_splitCurrencyId = m_account.currencyId();
 
-  // check if transaction is errnous or not
-  m_erronous = !m_transaction.splitSum().isZero();
+  // check if transaction is erroneous or not
+  m_erroneous = !m_transaction.splitSum().isZero();
 
   if (!m_uniqueId.isEmpty()) {
     m_uniqueId += '-';
@@ -242,8 +253,8 @@ bool Transaction::paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItem
   }
 
   // do we need to switch to the error color?
-  if (m_erronous) {
-    option.palette.setColor(QPalette::Text, KMyMoneyGlobalSettings::listErronousTransactionColor());
+  if (m_erroneous) {
+    option.palette.setColor(QPalette::Text, KMyMoneyGlobalSettings::listErroneousTransactionColor());
   }
 
   // do we need to switch to the negative balance color?
@@ -252,14 +263,14 @@ bool Transaction::paintRegisterCellSetup(QPainter *painter, QStyleOptionViewItem
     if (m_account.accountGroup() == MyMoneyAccount::Liability && !m_balance.isZero())
       showNegative = !showNegative;
     if (showNegative)
-      option.palette.setColor(QPalette::Text, KMyMoneyGlobalSettings::listErronousTransactionColor());
+      option.palette.setColor(QPalette::Text, KMyMoneyGlobalSettings::listErroneousTransactionColor());
   }
   return true;
 }
 
 void Transaction::registerCellText(QString& txt, int row, int col)
 {
-  int align = 0;
+  Qt::Alignment align;
   registerCellText(txt, align, row, col, 0);
 }
 
@@ -270,7 +281,7 @@ void Transaction::paintRegisterCell(QPainter *painter, QStyleOptionViewItemV4 &o
     const QStyle *style = option.widget ? option.widget->style() : QApplication::style();
     const QWidget* widget = option.widget;
 
-    // clear the mouse ove state before painting the background
+    // clear the mouse over state before painting the background
     option.state &= ~QStyle::State_MouseOver;
     // the background
     if (option.state & QStyle::State_Selected || option.state & QStyle::State_HasFocus) {
@@ -316,13 +327,30 @@ void Transaction::paintRegisterCell(QPainter *painter, QStyleOptionViewItemV4 &o
 
     // the text
     // construct the text for the cell
-    int align = Qt::AlignVCenter;
     QString txt;
+    option.displayAlignment = Qt::AlignVCenter;
     if (m_transaction != MyMoneyTransaction() && !m_inRegisterEdit) {
-      registerCellText(txt, align, index.row() - startRow(), index.column(), painter);
+      registerCellText(txt, option.displayAlignment, index.row() - startRow(), index.column(), painter);
+    }
+
+    if (Qt::mightBeRichText(txt)) {
+      QTextDocument document;
+      // this should set the alignment of the html but it does not work so htmls will be left aligned
+      document.setDefaultTextOption(QTextOption(option.displayAlignment));
+      document.setDocumentMargin(2);
+      document.setHtml(txt);
+      painter->translate(option.rect.topLeft());
+      QAbstractTextDocumentLayout::PaintContext ctx;
+      ctx.palette = option.palette;
+      // Highlighting text if item is selected
+      if (m_selected)
+        ctx.palette.setColor(QPalette::Text, option.palette.color(QPalette::HighlightedText));
+      document.documentLayout()->draw(painter, ctx);
+      painter->translate(-option.rect.topLeft());
+    } else {
+      // draw plain text properly aligned
+      style->drawItemText(painter, option.rect.adjusted(2, 0, -2, 0), option.displayAlignment, option.palette, true, txt, m_selected ? QPalette::HighlightedText : QPalette::Text);
     }
-    // adjust the text rectangle to obtain a reasonable spacing between the text and the grid
-    style->drawItemText(painter, option.rect.adjusted(2, 0, -2, 0), align, option.palette, true, txt, m_selected ? QPalette::HighlightedText : QPalette::Text);
 
     // draw the grid if it's needed
     if (KMyMoneySettings::showGrid()) {
@@ -338,7 +366,7 @@ void Transaction::paintRegisterCell(QPainter *painter, QStyleOptionViewItemV4 &o
 
     // possible icons
     if (index.row() == startRow() && index.column() == DetailColumn) {
-      if (m_erronous) {
+      if (m_erroneous) {
         QPixmap attention;
         attention.loadFromData(attentionSign, sizeof(attentionSign), 0, 0);
         style->drawItemPixmap(painter, option.rect, Qt::AlignRight | Qt::AlignVCenter, attention);
@@ -405,7 +433,7 @@ void Transaction::paintFormCell(QPainter *painter, const QStyleOptionViewItem &o
   painter->setPen(option.palette.text().color());
 
   QString txt;
-  int align = Qt::AlignVCenter;
+  Qt::Alignment align = Qt::AlignVCenter;
   bool editField = formCellText(txt, align, index.row(), index.column(), painter);
 
   // if we have an editable field and don't currently edit the transaction
@@ -498,7 +526,7 @@ bool Transaction::maybeTip(const QPoint& cpos, int row, int col, QRect& r, QStri
   if (col != DetailColumn)
     return false;
 
-  if (!m_erronous && m_transaction.splitCount() < 3)
+  if (!m_erroneous && m_transaction.splitCount() < 3)
     return false;
 
   // check for detail column in row 0 of the transaction for a possible
@@ -506,7 +534,7 @@ bool Transaction::maybeTip(const QPoint& cpos, int row, int col, QRect& r, QStri
   // the modelindex is based 1, so we need to add one here
   r = m_parent->visualRect(m_parent->model()->index(m_startRow + 1, col));
   r.setBottom(r.bottom() + (numRowsRegister() - 1)*r.height());
-  if (r.contains(cpos) && m_erronous) {
+  if (r.contains(cpos) && m_erroneous) {
     if (m_transaction.splits().count() < 2) {
       msg = QString("<qt>%1</qt>").arg(i18n("Transaction is missing a category assignment."));
     } else {
@@ -599,33 +627,74 @@ int Transaction::rowHeightHint(void) const
 }
 
 
-bool Transaction::matches(const QString& txt) const
+bool Transaction::matches(const RegisterFilter& filter) const
 {
-  if (txt.isEmpty() || m_transaction.splitCount() == 0)
+  // check if the state matches
+  if (!transaction().id().isEmpty()) {
+    switch (filter.state) {
+      default:
+        break;
+      case RegisterFilter::Imported:
+        if (!transaction().isImported())
+          return false;
+        break;
+      case RegisterFilter::Matched:
+        if (!split().isMatched())
+          return false;
+        break;
+      case RegisterFilter::Erroneous:
+        if (transaction().splitSum().isZero())
+          return false;
+        break;
+      case RegisterFilter::NotMarked:
+        if (split().reconcileFlag() != MyMoneySplit::NotReconciled)
+          return false;
+        break;
+      case RegisterFilter::NotReconciled:
+        if (split().reconcileFlag() != MyMoneySplit::NotReconciled
+            && split().reconcileFlag() != MyMoneySplit::Cleared)
+          return false;
+        break;
+      case RegisterFilter::Cleared:
+        if (split().reconcileFlag() != MyMoneySplit::Cleared)
+          return false;
+        break;
+    }
+  }
+
+  // check if the text matches
+  if (filter.text.isEmpty() || m_transaction.splitCount() == 0)
     return true;
 
   MyMoneyFile* file = MyMoneyFile::instance();
-  QString s(txt);
-  s.replace(MyMoneyMoney::thousandSeparator(), QChar());
 
   const QList<MyMoneySplit>&list = m_transaction.splits();
   QList<MyMoneySplit>::const_iterator it_s;
   for (it_s = list.begin(); it_s != list.end(); ++it_s) {
     // check if the text is contained in one of the fields
-    // memo, number, payee, account
-    if ((*it_s).memo().contains(txt, Qt::CaseInsensitive)
-        || (*it_s).number().contains(txt, Qt::CaseInsensitive))
+    // memo, number, payee, tag, account
+    if ((*it_s).memo().contains(filter.text, Qt::CaseInsensitive)
+        || (*it_s).number().contains(filter.text, Qt::CaseInsensitive))
       return true;
 
     if (!(*it_s).payeeId().isEmpty()) {
       const MyMoneyPayee& payee = file->payee((*it_s).payeeId());
-      if (payee.name().contains(txt, Qt::CaseInsensitive))
+      if (payee.name().contains(filter.text, Qt::CaseInsensitive))
         return true;
     }
+    if (!(*it_s).tagIdList().isEmpty()) {
+      const QList<QString>& t = (*it_s).tagIdList();
+      for (int i = 0; i < t.count(); i++) {
+        if ((file->tag(t[i])).name().contains(filter.text, Qt::CaseInsensitive))
+          return true;
+      }
+    }
     const MyMoneyAccount& acc = file->account((*it_s).accountId());
-    if (acc.name().contains(txt, Qt::CaseInsensitive))
+    if (acc.name().contains(filter.text, Qt::CaseInsensitive))
       return true;
 
+    QString s(filter.text);
+    s.replace(MyMoneyMoney::thousandSeparator(), QChar());
     if (!s.isEmpty()) {
       // check if any of the value field matches if a value has been entered
       QString r = (*it_s).value().formatMoney(m_account.fraction(), false);
@@ -717,9 +786,8 @@ StdTransaction::StdTransaction(Register *parent, const MyMoneyTransaction& trans
         setupFormHeader(m_transaction.splitByAccount(m_split.accountId(), false).accountId());
         break;
     }
-  } catch (MyMoneyException *e) {
-    kDebug(2) << "Problem determining the category for transaction '" << m_transaction.id() << "'. Reason: " << e->what()  << "\n";
-    delete e;
+  } catch (const MyMoneyException &e) {
+    kDebug(2) << "Problem determining the category for transaction '" << m_transaction.id() << "'. Reason: " << e.what()  << "\n";
   }
   m_rowsForm = 6;
 
@@ -811,7 +879,7 @@ void StdTransaction::loadTab(TransactionForm* form)
 void StdTransaction::setupForm(TransactionForm* form)
 {
   Transaction::setupForm(form);
-  form->setSpan(3, ValueColumn1, 3, 1);
+  form->setSpan(4, ValueColumn1, 3, 1);
 }
 
 bool StdTransaction::showRowInForm(int row) const
@@ -825,7 +893,7 @@ void StdTransaction::setShowRowInForm(int row, bool show)
     m_showAccountRow = show;
 }
 
-bool StdTransaction::formCellText(QString& txt, int& align, int row, int col, QPainter* /* painter */)
+bool StdTransaction::formCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* /* painter */)
 {
   // if(m_transaction != MyMoneyTransaction()) {
   switch (row) {
@@ -897,15 +965,18 @@ bool StdTransaction::formCellText(QString& txt, int& align, int row, int col, QP
       switch (col) {
         case LabelColumn1:
           align |= Qt::AlignLeft;
-          txt = i18n("Memo");
+          txt = i18n("Tags");
           break;
 
         case ValueColumn1:
-          align &= ~Qt::AlignVCenter;
-          align |= Qt::AlignTop;
           align |= Qt::AlignLeft;
-          if (m_transaction != MyMoneyTransaction())
-            txt = m_split.memo().section('\n', 0, 2);
+          if (!m_tagList.isEmpty()) {
+            for (int i = 0; i < m_tagList.size() - 1; i++)
+              txt += m_tagList[i] + ", ";
+            txt += m_tagList.last();
+          }
+          //if (m_transaction != MyMoneyTransaction())
+          //  txt = m_split.tagId();
           break;
 
         case LabelColumn2:
@@ -922,6 +993,23 @@ bool StdTransaction::formCellText(QString& txt, int& align, int row, int col, QP
       }
       break;
 
+    case 4:
+      switch (col) {
+        case LabelColumn1:
+          align |= Qt::AlignLeft;
+          txt = i18n("Memo");
+          break;
+
+        case ValueColumn1:
+          align &= ~Qt::AlignVCenter;
+          align |= Qt::AlignTop;
+          align |= Qt::AlignLeft;
+          if (m_transaction != MyMoneyTransaction())
+            txt = m_split.memo().section('\n', 0, 2);
+          break;
+      }
+      break;
+
     case 5:
       switch (col) {
         case LabelColumn2:
@@ -940,10 +1028,10 @@ bool StdTransaction::formCellText(QString& txt, int& align, int row, int col, QP
   if (col == ValueColumn2 && row == 1) {
     return haveNumberField();
   }
-  return (col == ValueColumn1 && row < 4) || (col == ValueColumn2 && row > 0 && row != 4);
+  return (col == ValueColumn1 && row < 5) || (col == ValueColumn2 && row > 0 && row != 4);
 }
 
-void StdTransaction::registerCellText(QString& txt, int& align, int row, int col, QPainter* painter)
+void StdTransaction::registerCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* painter)
 {
   switch (row) {
     case 0:
@@ -966,6 +1054,13 @@ void StdTransaction::registerCellText(QString& txt, int& align, int row, int col
               break;
             case AccountFirst:
               txt = m_category;
+              if (!m_tagList.isEmpty()) {
+                txt += " ( ";
+                for (int i = 0; i < m_tagList.size() - 1; i++) {
+                  txt += "<span style='color: " + m_tagColorList[i].name() + "'>&#x25CF;</span> " + m_tagList[i] + ", ";
+                }
+                txt += "<span style='color: " + m_tagColorList.last().name() + "'>&#x25CF;</span> " + m_tagList.last() + " )";
+              }
               break;
           }
           align |= Qt::AlignLeft;
@@ -979,7 +1074,7 @@ void StdTransaction::registerCellText(QString& txt, int& align, int row, int col
               if (txt.isEmpty() && !m_split.value().isZero()) {
                 txt = i18n("*** UNASSIGNED ***");
                 if (painter)
-                  painter->setPen(KMyMoneyGlobalSettings::listErronousTransactionColor());
+                  painter->setPen(KMyMoneyGlobalSettings::listErroneousTransactionColor());
               }
             }
           }
@@ -1028,6 +1123,13 @@ void StdTransaction::registerCellText(QString& txt, int& align, int row, int col
           switch (m_parent->getDetailsColumnType()) {
             case PayeeFirst:
               txt = m_category;
+              if (!m_tagList.isEmpty()) {
+                txt += " ( ";
+                for (int i = 0; i < m_tagList.size() - 1; i++) {
+                  txt += "<span style='color: " + m_tagColorList[i].name() + "'>&#x25CF;</span> " + m_tagList[i] + ", ";
+                }
+                txt += "<span style='color: " + m_tagColorList.last().name() + "'>&#x25CF;</span> " + m_tagList.last() + " )";
+              }
               break;
             case AccountFirst:
               txt = m_payee;
@@ -1037,7 +1139,7 @@ void StdTransaction::registerCellText(QString& txt, int& align, int row, int col
           if (txt.isEmpty() && !m_split.value().isZero()) {
             txt = i18n("*** UNASSIGNED ***");
             if (painter)
-              painter->setPen(KMyMoneyGlobalSettings::listErronousTransactionColor());
+              painter->setPen(KMyMoneyGlobalSettings::listErroneousTransactionColor());
           }
           break;
 
@@ -1067,7 +1169,7 @@ int StdTransaction::registerColWidth(int col, const QFontMetrics& cellFontMetric
 
   int nw = 0;
   for (int i = firstRow; i <= lastRow; ++i) {
-    int align;
+    Qt::Alignment align;
     registerCellText(txt, align, i, col, 0);
     int w = cellFontMetrics.width(txt + "   ");
     if (w > nw)
@@ -1089,8 +1191,10 @@ void StdTransaction::arrangeWidgetsInForm(QMap<QString, QWidget*>& editWidgets)
   arrangeWidget(m_form, 1, ValueColumn1, editWidgets["payee"]);
   arrangeWidget(m_form, 2, LabelColumn1, editWidgets["category-label"]);
   arrangeWidget(m_form, 2, ValueColumn1, editWidgets["category"]->parentWidget());
-  arrangeWidget(m_form, 3, LabelColumn1, editWidgets["memo-label"]);
-  arrangeWidget(m_form, 3, ValueColumn1, editWidgets["memo"]);
+  arrangeWidget(m_form, 3, LabelColumn1, editWidgets["tag-label"]);
+  arrangeWidget(m_form, 3, ValueColumn1, editWidgets["tag"]);
+  arrangeWidget(m_form, 4, LabelColumn1, editWidgets["memo-label"]);
+  arrangeWidget(m_form, 4, ValueColumn1, editWidgets["memo"]);
   if (haveNumberField()) {
     arrangeWidget(m_form, 1, LabelColumn2, editWidgets["number-label"]);
     arrangeWidget(m_form, 1, ValueColumn2, editWidgets["number"]);
@@ -1108,12 +1212,15 @@ void StdTransaction::arrangeWidgetsInForm(QMap<QString, QWidget*>& editWidgets)
     KMyMoneyCombo* combo = dynamic_cast<KMyMoneyCombo*>(*it);
     kMyMoneyLineEdit* edit = dynamic_cast<kMyMoneyLineEdit*>(*it);
     KMyMoneyPayeeCombo* payee = dynamic_cast<KMyMoneyPayeeCombo*>(*it);
+    KTagContainer* tag = dynamic_cast<KTagContainer*>(*it);
     if (combo)
       combo->setClickMessage(QString());
     if (edit)
       edit->setClickMessage(QString());
     if (payee)
       payee->setClickMessage(QString());
+    if (tag)
+      tag->tagCombo()->setClickMessage(QString());
   }
 
   KMyMoneyTransactionForm::TransactionForm* form = dynamic_cast<KMyMoneyTransactionForm::TransactionForm*>(m_form);
@@ -1147,8 +1254,10 @@ void StdTransaction::tabOrderInForm(QWidgetList& tabOrderWidgets) const
       w = w->findChild<KPushButton*>("splitButton");
       if (w)
         tabOrderWidgets.append(w);
-    } else if (*it_s == "memo") {
+    } else if (*it_s == "tag") {
       tabOrderWidgets.append(focusWidget(m_form->cellWidget(3, ValueColumn1)));
+    } else if (*it_s == "memo") {
+      tabOrderWidgets.append(focusWidget(m_form->cellWidget(4, ValueColumn1)));
     } else if (*it_s == "number") {
       if (haveNumberField()) {
         if ((w = focusWidget(m_form->cellWidget(1, ValueColumn2))))
@@ -1178,12 +1287,13 @@ void StdTransaction::arrangeWidgetsInRegister(QMap<QString, QWidget*>& editWidge
   arrangeWidget(m_parent, m_startRow + 1, DateColumn, editWidgets["status"]);
   arrangeWidget(m_parent, m_startRow + 0, DetailColumn, editWidgets["payee"]);
   arrangeWidget(m_parent, m_startRow + 1, DetailColumn, editWidgets["category"]->parentWidget());
-  arrangeWidget(m_parent, m_startRow + 2, DetailColumn, editWidgets["memo"]);
+  arrangeWidget(m_parent, m_startRow + 2, DetailColumn, editWidgets["tag"]);
+  arrangeWidget(m_parent, m_startRow + 3, DetailColumn, editWidgets["memo"]);
   arrangeWidget(m_parent, m_startRow + 0, PaymentColumn, editWidgets["payment"]);
   arrangeWidget(m_parent, m_startRow + 0, DepositColumn, editWidgets["deposit"]);
 
   // increase the height of the row containing the memo widget
-  m_parent->setRowHeight(m_startRow + 2, m_parent->rowHeightHint() * 3);
+  m_parent->setRowHeight(m_startRow + 3, m_parent->rowHeightHint() * 3);
 }
 
 void StdTransaction::tabOrderInRegister(QWidgetList& tabOrderWidgets) const
@@ -1211,8 +1321,10 @@ void StdTransaction::tabOrderInRegister(QWidgetList& tabOrderWidgets) const
       w = w->findChild<KPushButton*>("splitButton");
       if (w)
         tabOrderWidgets.append(w);
-    } else if (*it_s == "memo") {
+    } else if (*it_s == "tag") {
       tabOrderWidgets.append(focusWidget(m_parent->cellWidget(m_startRow + 2, DetailColumn)));
+    } else if (*it_s == "memo") {
+      tabOrderWidgets.append(focusWidget(m_parent->cellWidget(m_startRow + 3, DetailColumn)));
     } else if (*it_s == "payment") {
       tabOrderWidgets.append(focusWidget(m_parent->cellWidget(m_startRow + 0, PaymentColumn)));
     } else if (*it_s == "deposit") {
@@ -1228,8 +1340,10 @@ int StdTransaction::numRowsRegister(bool expanded) const
 {
   int numRows = 1;
   if (expanded) {
-    numRows = 3;
+    numRows = 4;
     if (!m_inEdit) {
+      //When not in edit Tags haven't a separate row;
+      numRows--;
       if (m_payee.isEmpty()) {
         numRows--;
       }
@@ -1353,7 +1467,7 @@ void InvestTransaction::activity(QString& txt, MyMoneySplit::investTransactionTy
   }
 }
 
-bool InvestTransaction::formCellText(QString& txt, int& align, int row, int col, QPainter* /* painter */)
+bool InvestTransaction::formCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* /* painter */)
 {
   bool fieldEditable = false;
 
@@ -1467,7 +1581,7 @@ bool InvestTransaction::formCellText(QString& txt, int& align, int row, int col,
         case LabelColumn2:
           align |= Qt::AlignLeft;
           if (haveFees() && !m_feeCategory.isEmpty())
-            txt = i18n("Fee amount");
+            txt = i18n("Fee Amount");
           break;
 
         case ValueColumn2:
@@ -1561,7 +1675,7 @@ bool InvestTransaction::formCellText(QString& txt, int& align, int row, int col,
   return fieldEditable;
 }
 
-void InvestTransaction::registerCellText(QString& txt, int& align, int row, int col, QPainter* /* painter */)
+void InvestTransaction::registerCellText(QString& txt, Qt::Alignment& align, int row, int col, QPainter* /* painter */)
 {
   switch (row) {
     case 0:
